unit d4dnDevGuide.StateCollection;
interface
uses
  System.Collections;

type

  // Define a class to store in the Collection
  TStateInfo = class
    StateName: String;
    StateCapital: String;
    Admission: DateTime;
    constructor Create(aName, aCapital: String; aAddmision: DateTime);
  end;

  // Define the strongly-typed collection
  TStateInfoCollection = class(CollectionBase)
  private
    function GetStateInfo(Index: Integer): TStateInfo;
    procedure SetStateInfo(Index: Integer; const Value: TStateInfo);
    procedure VerifyType(Value: TObject);
  strict protected
    // Type checking events
    procedure OnInsert(index: integer; value: TObject); override;
    procedure OnSet(index: integer; oldValue: TObject;
      newValue: TObject); override;
    procedure OnValidate(value: TObject); override;
  public
    constructor Create;
    function Add(value: TStateInfo): Integer;
    function IndexOf(value: TStateInfo): Integer;
    procedure Insert(index: integer; value: TStateInfo);
    procedure Remove(value: TStateInfo);
    function Contains(value: TStateInfo): Boolean;
    procedure PrintItems;
    property StateInfo[Index: Longint]: TStateInfo read GetStateInfo
      write SetStateInfo;
  end;

implementation

{ TStateInfo }
constructor TStateInfo.Create(AName, ACapital: String;
  AAddmision: DateTime);
begin
  inherited Create();
  StateName    := AName;
  StateCapital := ACapital;
  Admission   := AAddmision;
end;

{ TStateInfoCollection }

constructor TStateInfoCollection.Create;
begin
  inherited Create;
end;

function TStateInfoCollection.GetStateInfo(Index: Integer): TStateInfo;
begin
  Result := List[Index] as TStateInfo;
end;

procedure TStateInfoCollection.SetStateInfo(Index: Integer;
  const Value: TStateInfo);
begin
  List[Index] := Value;
end;

procedure TStateInfoCollection.OnInsert(Index: integer; Value: TObject);
begin
  VerifyType(Value);
end;

procedure TStateInfoCollection.OnSet(Index: integer; OldValue:
  TObject; NewValue: TObject);
begin
  VerifyType(NewValue);
end;

procedure TStateInfoCollection.OnValidate(Value: TObject);
begin
  VerifyType(Value);
end;

function TStateInfoCollection.Add(Value: TStateInfo): Integer;
begin
  Result := List.Add(Value);
end;

function TStateInfoCollection.IndexOf(Value: TStateInfo): Integer;
begin
  Result := List.IndexOf(Value);
end;

procedure TStateInfoCollection.Insert(Index: integer; Value: TStateInfo);
begin
  List.Insert(Index, Value);
end;

procedure TStateInfoCollection.Remove(Value: TStateInfo);
begin
  List.Remove(Value);
end;

function TStateInfoCollection.Contains(Value: TStateInfo): Boolean;
begin
  result := List.Contains(Value);
end;

procedure TStateInfoCollection.PrintItems;
const
  fmt = 'State: {0}, Capital: {1}, Admission: {2}';
var
  i: Integer;
  si: TStateInfo;
begin
  for i := 0 to Count -1 do
  begin
    si := TStateInfo(List[i]);
    System.Console.WriteLine(System.String.Format(fmt, [si.StateName,
      si.StateCapital, si.Admission.ToShortDateString()]));
  end;
end;

procedure TStateInfoCollection.VerifyType(Value: TObject);
begin
  if not (Value is TStateInfo) then
    raise ArgumentException.Create('Invalid Type');
end;

end.
