unit d4dnDevGuide.StateDictionary;
interface

uses
  System.Collections;

type

  // Define a class to store in the dictionary
  TStateInfo = class
    StateName: String;
    StateCapital: String;
    Admission: DateTime;
    constructor Create(AName, ACapital: String; AAddmision: DateTime);
  end;

  // Define the strongly-typed dictionary
  TStateInfoDictionary = class(DictionaryBase)
  private
    function GetStateInfo(key: String): TStateInfo;
    procedure SetStateInfo(key: String; const Value: TStateInfo);
    function GetKeys: ICollection;
    function GetValues: ICollection;
    procedure VerifyType(Value: TObject);
  strict protected
    // Type checking events
    procedure OnInsert(key: TObject; value: TObject); override;
    procedure OnSet(Key: TObject; OldValue: TObject;
      NewValue: TObject); override;
    procedure OnValidate(key: TObject; value: TObject); override;
  public
    constructor Create;
    procedure Add(key: String; value: TStateInfo);
    procedure Remove(key: String);
    function Contains(key: String): Boolean;

    procedure PrintItems;

    property StateInfo[key: String]: TStateInfo read GetStateInfo
      write SetStateInfo;
    property Keys: ICollection read GetKeys;
    property Values: ICollection read GetValues;
  end;

implementation

{ TStateInfo }
constructor TStateInfo.Create(AName, ACapital: String;
  AAddmision: DateTime);
begin
  inherited Create();
  StateName    := AName;
  StateCapital := ACapital;
  Admission   := AAddmision;
end;

{ TStateInfoDictionary }

constructor TStateInfoDictionary.Create;
begin
  inherited Create;
end;

procedure TStateInfoDictionary.OnInsert(key: TObject; value: TObject);
begin
  VerifyType(Value);
end;

procedure TStateInfoDictionary.OnSet(Key: TObject; OldValue:
  TObject; NewValue: TObject);
begin
  VerifyType(NewValue);
end;

procedure TStateInfoDictionary.OnValidate(key: TObject; Value: TObject);
begin
  VerifyType(Value);
end;

procedure TStateInfoDictionary.Add(key: String; value: TStateInfo);
begin
  Dictionary.Add(key, value);
end;

procedure TStateInfoDictionary.Remove(key: String);
begin
  Dictionary.Remove(key);
end;

function TStateInfoDictionary.Contains(key: String): Boolean;
begin
  result := Dictionary.Contains(key);
end;

procedure TStateInfoDictionary.PrintItems;
const
  fmt = 'State: {0}, Capital: {1}, Admission: {2}';
var
  si: TStateInfo;
  enum: IDictionaryEnumerator;
begin
  enum := GetEnumerator;
  while enum.MoveNext() do
  begin
    si := TStateInfo(enum.Value);
    writeln(System.String.Format(fmt, [si.StateName, si.StateCapital,
      si.Admission.ToShortDateString()]));
  end;
end;

function TStateInfoDictionary.GetStateInfo(key: String): TStateInfo;
begin
  Result := Dictionary[key] as TStateInfo;
end;

procedure TStateInfoDictionary.SetStateInfo(key: String;
  const Value: TStateInfo);
begin
  Dictionary[key] := Value;
end;

function TStateInfoDictionary.GetKeys: ICollection;
begin
  Result := Dictionary.Keys;
end;

function TStateInfoDictionary.GetValues: ICollection;
begin
  Result := Dictionary.Values;
end;

procedure TStateInfoDictionary.VerifyType(Value: TObject);
begin
  if not (Value is TStateInfo) then
    raise ArgumentException.Create('Invalid Type');
end;

end.
