unit d8dg.Alarms;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms;

type
  AlarmEventArgs = class(EventArgs)
  public
    FTime: DateTime;
  public
    property Time: DateTime read FTime write FTime;
  end;

  AlarmHandler = procedure (Sender: TObject; e: AlarmEventArgs) of object;

  AlarmClock = class(System.ComponentModel.Component)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    FTimer: Timer;        // internal timer
    FAlarm: AlarmHandler; // alarm event
    FAlarmFired: Boolean;
    procedure TimerHandler(Sender: TObject; e: EventArgs); // tick handler
    function GetCurrentTime: DateTime;
    procedure SetEnabled(Value: Boolean);
    function GetEnabled: Boolean;
    procedure SetAlarmTime(const Value: DateTime);
  protected
    procedure OnAlarm(args: AlarmEventArgs); virtual;
    procedure Init; virtual;
  public
    FAlarmTime: DateTime;
    FEnabled: Boolean;
    constructor Create; overload;
    constructor Create(Container: System.ComponentModel.IContainer);
      overload;
    property CurrentTime: DateTime read GetCurrentTime;
  published
    property AlarmTime: DateTime read FAlarmTime write SetAlarmTime;
    property Enabled: Boolean read GetEnabled write SetEnabled;
    property Alarm: AlarmHandler add FAlarm remove FAlarm;
  end;

implementation

uses
  System.Globalization;

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure AlarmClock.InitializeComponent;
begin
  Self.Components := System.ComponentModel.Container.Create;
end;
{$ENDREGION}

constructor AlarmClock.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  Init;
end;

constructor AlarmClock.Create(Container:
  System.ComponentModel.IContainer);
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  Container.Add(Self);
  InitializeComponent;
  Init;
end;

procedure AlarmClock.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

procedure AlarmClock.TimerHandler(Sender: TObject; e: EventArgs);
var
  Now: DateTime;
  Args: AlarmEventArgs;
begin
  if not DesignMode then // don't fire in design mode
  begin
    Now := DateTime.Now;
    // compare Now to alarm time, ignoring milliseconds
    if (not FAlarmFired) and (Now >= FAlarmTime) then
    begin
      Args := AlarmEventArgs.Create;
      Args.Time := Now;
      OnAlarm(Args);
      FAlarmFired := True;
    end;
  end;
end;

function AlarmClock.GetEnabled: Boolean;
begin
  Result := FTimer.Enabled;
end;

procedure AlarmClock.SetEnabled(Value: Boolean);
begin
  FTimer.Enabled := Value;
  if Value then
    FAlarmFired := False;
end;

function AlarmClock.GetCurrentTime: DateTime;
begin
  Result := DateTime.Now;
end;

procedure AlarmClock.Init;
begin
  FTimer := Timer.Create;
  Self.Components.Add(FTimer);
  // tick every second
  FTimer.Interval := 1000;
  // listen to Timer's Tick event
  Include(FTimer.Tick, TimerHandler);
end;

procedure AlarmClock.OnAlarm(args: AlarmEventArgs);
begin
  if Assigned(FAlarm) then
    FAlarm(Self, args);
end;

procedure AlarmClock.SetAlarmTime(const Value: DateTime);
begin
  if FAlarmTime <> Value then
  begin
    FAlarmTime := Value;
    FAlarmFired := False;
  end;
end;

end.
