unit d8dg.Cards;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Threading;

type
  CardSuit = (CSClub, CSDiamond, CSHeart, CSSpade);

  CardValue = (CVAce, CVTwo, CVThree, CVFour, CVFive, CVSix, CVSeven,
    CVEight, CVNine, CVTen, CVJack, CVQueen, CVKing);

  /// <summary>
  /// Represents a single playing card, with  suit and value.
  /// </summary>
  [Serializable]
  Card = record
    cSuit: CardSuit;
    cValue: CardValue;
  end;

  /// <summary>
  /// Card exception class
  /// </summary>
  [Serializable]
    ECardException = class(Exception)
  end;

  [ToolboxItem(True)]
  PlayingCard = class(System.Windows.Forms.Control)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    class var
      FDrawBrush: SolidBrush;
      FBrushRef: Integer;
  private
    FBackPen: Pen;
    FBorderPen: Pen;
    FSymbolFont: Font;
    FSansSerifFont: Font;
    FCardBackBrush: SolidBrush;
    FSuit: CardSuit;
    FValue: CardValue;
    FFaceUp: Boolean;
    FBorderWidth: Integer;
    procedure SetSuit(Value: CardSuit);
    procedure SetValue(Value: CardValue);
    procedure SetFaceUp(Value: Boolean);
    procedure SetBorderWidth(Value: Integer);
  strict protected
    procedure InitComp;
    procedure OnPaint(e: PaintEventArgs); override;
    function get_DefaultSize: Size; override;
    property BorderWidth: Integer read FBorderWidth write SetBorderWidth;
  public
    constructor Create; overload;
    constructor Create(Container: System.ComponentModel.IContainer);
      overload;
  published
    [Description('The suit of the card'), Category('Appearance')]
    property Suit: CardSuit read FSuit write SetSuit;
    [Description('The face value of the card'), Category('Appearance')]
    property Value: CardValue read FValue write SetValue;
    [Description('Whether the card is face up or down'),
      Category('Appearance')]
    property FaceUp: Boolean read FFaceUp write SetFaceUp;
  end;

implementation

uses
  System.Globalization;

const
  SuitArray: array[0..3] of string = (Chr(167), Chr(168), Chr(169),
    Chr(170));
  ValueArray: array[0..12] of string = ('A', '2', '3', '4', '5', '6',
    '7', '8', '9', '10', 'J', 'Q', 'K');

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure PlayingCard.InitializeComponent;
begin
  Self.Components := System.ComponentModel.Container.Create;
end;
{$ENDREGION}

constructor PlayingCard.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  InitComp;
end;

constructor PlayingCard.Create(Container:
  System.ComponentModel.IContainer);
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  Container.Add(Self);
  InitializeComponent;
  InitComp;
end;

procedure PlayingCard.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    FBackPen.Dispose;
    FBorderPen.Dispose;
    FCardBackBrush.Dispose;
    FSymbolFont.Dispose;
    FSansSerifFont.Dispose;
    if System.Threading.Interlocked.Decrement(FBrushRef) = 0 then
    begin
      FDrawBrush.Dispose();
      FDrawBrush := nil;
    end;
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

procedure PlayingCard.InitComp;
begin
  FBorderWidth := 4;
  FFaceUp := True;
  SetStyle(ControlStyles.UserPaint, True);
  SetStyle(ControlStyles.AllPaintingInWmPaint, True);
  SetStyle(ControlStyles.DoubleBuffer, True);
  SetStyle(ControlStyles.ResizeRedraw, True);
  SetStyle(ControlStyles.StandardClick, True);
  SetStyle(ControlStyles.StandardDoubleClick, True);
  SetStyle(ControlStyles.Opaque, True);
  FBackPen := Pen.Create(BackColor, FBorderWidth);
  FBorderPen := Pen.Create(Color.FromKnownColor(KnownColor.Black), 1);
  FCardBackBrush := SolidBrush.Create(ForeColor);
  FSymbolFont := System.Drawing.Font.Create('Symbol', 36);
  FSansSerifFont := System.Drawing.Font.Create('Arial', 24);
  if System.Threading.Interlocked.Increment(FBrushRef) = 1 then
    FDrawBrush := SolidBrush.Create(Color.FromKnownColor(KnownColor.Red));
  System.Threading.Interlocked.Increment(FBrushRef);
  ForeColor := Color.FromKnownColor(KnownColor.White);
  BackColor := Color.FromKnownColor(KnownColor.Gray);
end;

procedure PlayingCard.OnPaint(e: PaintEventArgs);
var
  penWidth: Double;
  rect: Rectangle;
  fmt: StringFormat;
begin
  rect := ClientRectangle;
  FDrawBrush.Color := Color.FromKnownColor(KnownColor.White);
	e.Graphics.FillRectangle(FDrawBrush, rect);
	rect.Inflate(FBorderWidth * -1, FBorderWidth * -1);
	if FFaceUp then  // card face up
  begin
    // get color from suit
    if (FSuit = CardSuit.CSDiamond) or (FSuit = CardSuit.CSHeart) then
      FDrawBrush.Color := Color.FromKnownColor(KnownColor.Red)
    else
      FDrawBrush.Color := Color.FromKnownColor(KnownColor.Black);
    fmt := StringFormat.Create;
    // Draw suit character
    fmt.Alignment := StringAlignment.Center;
    fmt.LineAlignment := StringAlignment.Center;
    e.Graphics.DrawString(SuitArray[Ord(FSuit)], FSymbolFont,
      FDrawBrush, rect, fmt);
    // Draw top/left value
    fmt.Alignment := StringAlignment.Near;
    fmt.LineAlignment := StringAlignment.Near;
    e.Graphics.DrawString(ValueArray[Ord(FValue)], FSansSerifFont,
      FDrawBrush, rect, fmt);
    // Draw bottom/right value
    fmt.Alignment := StringAlignment.Far;
    fmt.LineAlignment := StringAlignment.Far;
    e.Graphics.DrawString(ValueArray[Ord(FValue)], FSansSerifFont,
      FDrawBrush, rect, fmt);
  end
  else begin // card face down
    // draw background image
    if BackgroundImage <> nil then
      e.Graphics.DrawImage(BackgroundImage,
        (Width - BackgroundImage.Width) div 2,
        (Height - BackgroundImage.Height) div 2);
    // draw inset border
    FBackPen.Color := BackColor;
    e.Graphics.DrawRectangle(FBackPen, rect);
    end;
  // draw outter border
  penWidth := FBorderPen.Width;
  e.Graphics.DrawRectangle(FBorderPen, 0, 0, Width - penWidth,
    Height - penWidth);
end;

function PlayingCard.get_DefaultSize: Size;
begin
  Result := System.Drawing.Size.Create(90, 120);
end;

procedure PlayingCard.SetBorderWidth(Value: Integer);
begin
  if FBorderWidth <> Value then
  begin
    FBorderWidth := Value;
    Invalidate();
  end;
end;

procedure PlayingCard.SetSuit(Value: CardSuit);
begin
  if not Enum.IsDefined(TypeOf(CardSuit), Suit) then
    raise ArgumentOutOfRangeException.Create;
  if FSuit <> Value then
  begin
    FSuit := Value;
    Invalidate;
  end;
end;

procedure PlayingCard.SetValue(Value: CardValue);
begin
  if not Enum.IsDefined(TypeOf(CardValue), Value) then
    raise ArgumentOutOfRangeException.Create;
  if FValue <> Value then
  begin
    FValue := Value;
    Invalidate;
  end;
end;

procedure PlayingCard.SetFaceUp(Value: Boolean);
begin
  if FFaceUp <> Value then
  begin
    FFaceUp := Value;
    Invalidate;
  end;
end;

end.
