unit d8dg.ExplorerCtl;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.IO, System.Runtime.InteropServices;

type
  ExplorerViewerEventArgs = class(System.EventArgs)
  private
    FActiveFile: string;
  public
    /// <summary>
    /// Active file at time of event invocation.
    /// </summary>
    property ActiveFile: string read FActiveFile write FActiveFile;
  end;

  ExplorerViewerEvent = procedure (Sender: System.Object;
    Args: ExplorerViewerEventArgs) of object;

  ExplorerViewer = class(System.Windows.Forms.UserControl)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    components: System.ComponentModel.IContainer;
    ListView: System.Windows.Forms.ListView;
    Splitter: System.Windows.Forms.Splitter;
    TreeView: System.Windows.Forms.TreeView;
    ImageList: System.Windows.Forms.ImageList;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
    procedure TreeView_AfterSelect(sender: System.Object; e: System.Windows.Forms.TreeViewEventArgs);
    procedure TreeView_BeforeExpand(sender: System.Object; e: System.Windows.Forms.TreeViewCancelEventArgs);
    procedure ListView_ItemActivate(sender: System.Object; e: System.EventArgs);
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    FIconList: ArrayList;
    FAutoActivate: Boolean;
    FFileActivated: ExplorerViewerEvent;
    function GetCurrentItem: string;
    procedure SetCurrentItem(const Value: string);
    procedure DisposeIcons;
    function IncludeBackslash(APath: string): string;
    function ExcludeBackslash(APath: string): string;
  protected
    /// <doc><desc>
    /// Adds a node to the TreeView with given parent.
    /// </desc></doc>
    procedure AddTreeNode(Parent: TreeNode; NodeName: string); virtual;
    /// <doc><desc>
    /// Fills ListView with files from given directory.
    /// </desc></doc>
    procedure FillListView(APath: string); virtual;
    /// <doc><desc>
    /// Initializes TreeView will logical drives.
    /// </desc></doc>
    procedure FillTreeView; virtual;
    /// <doc><desc>
    /// Gets list of subdirectories for a node and creates subnodes.
    /// </desc></doc>
    procedure RefreshNode(Node: TreeNode); virtual;
    /// <doc><desc>
    /// Fires FileActivated event.
    /// </desc></doc>
    procedure OnFileActivated(Args: ExplorerViewerEventArgs); virtual;
    /// <doc><desc>
    /// Activates the specified file by using the shell to execute.
    /// </desc></doc>
    procedure ActivateFile(FileName: string); virtual;
  public
    constructor Create;
  published
    /// <doc><desc>
    /// Specifies whether ActiveFile should be called when file is
    /// activated in the ListView.
    /// </desc></doc>
    property AutoActivate: Boolean read FAutoActivate
      write FAutoActivate default True;
    /// <doc><desc>
    /// Full path name of currently selected file.
    /// </desc></doc>
    property CurrentItem: string read GetCurrentItem write SetCurrentItem;
    /// <doc><desc>
    /// Event that fires when item is activated in ListView.
    /// </desc></doc>
    property FileActivated: ExplorerViewerEvent add FFileActivated
      remove FFileActivated;
  end;

  ExplorerViewerError = class(Exception)
  end;

  [assembly: RuntimeRequiredAttribute(TypeOf(ExplorerViewer))]

function ExtractIcon(FileName: string): System.Drawing.Icon;

implementation

uses
  System.Globalization, System.Diagnostics;

{ Icon extraction support }

type
  [StructLayout(LayoutKind.Sequential)]
  SHFILEINFO = record
    hIcon: IntPtr;
    iIcon: Integer;
    dwAttributes: Cardinal;
    [MarshalAs(UnmanagedType.ByValTStr, SizeConst=260)]
    szDisplayName: string;
    [MarshalAs(UnmanagedType.ByValTStr, SizeConst=80)]
    szTypeName: string;
  end;

  SHGFI = (SmallIcon = $00000001, LargeIcon = $00000000,
    Icon = $00000100, DisplayName = $00000200, Typename = $00000400,
    SysIconIndex = $00004000, UseFileAttributes = $00000010);

[DllImport('Shell32.dll')]
function SHGetFileInfo(pszPath: string; dwFileAttributes: Cardinal;
  out psfi: SHFILEINFO; cbfileInfo: Cardinal; uFlags: SHGFI): Integer;
  external;

function ExtractIcon(FileName: string): System.Drawing.Icon;
var
  SHFI: SHFILEINFO;
begin
  if SHGetFileInfo(FileName, 0, SHFI, Marshal.SizeOf(SHFI),
    SHGFI.Icon or SHGFI.LargeIcon) <> 0 then
    Result := System.Drawing.Icon.FromHandle(SHFI.hIcon)
  else
    Result := nil;
end;

{ Explorerview }

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support -- do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure ExplorerViewer.InitializeComponent;
begin
  Self.components := System.ComponentModel.Container.Create;
  Self.ListView := System.Windows.Forms.ListView.Create;
  Self.ImageList := System.Windows.Forms.ImageList.Create(Self.components);
  Self.Splitter := System.Windows.Forms.Splitter.Create;
  Self.TreeView := System.Windows.Forms.TreeView.Create;
  Self.SuspendLayout;
  // 
  // ListView
  // 
  Self.ListView.Dock := System.Windows.Forms.DockStyle.Right;
  Self.ListView.LargeImageList := Self.ImageList;
  Self.ListView.Location := System.Drawing.Point.Create(95, 0);
  Self.ListView.Name := 'ListView';
  Self.ListView.Size := System.Drawing.Size.Create(121, 150);
  Self.ListView.TabIndex := 0;
  Include(Self.ListView.ItemActivate, Self.ListView_ItemActivate);
  // 
  // ImageList
  // 
  Self.ImageList.ImageSize := System.Drawing.Size.Create(16, 16);
  Self.ImageList.TransparentColor := System.Drawing.Color.Transparent;
  // 
  // Splitter
  // 
  Self.Splitter.Dock := System.Windows.Forms.DockStyle.Right;
  Self.Splitter.Location := System.Drawing.Point.Create(92, 0);
  Self.Splitter.Name := 'Splitter';
  Self.Splitter.Size := System.Drawing.Size.Create(3, 150);
  Self.Splitter.TabIndex := 1;
  Self.Splitter.TabStop := False;
  // 
  // TreeView
  // 
  Self.TreeView.Dock := System.Windows.Forms.DockStyle.Fill;
  Self.TreeView.FullRowSelect := True;
  Self.TreeView.ImageIndex := -1;
  Self.TreeView.Location := System.Drawing.Point.Create(0, 0);
  Self.TreeView.Name := 'TreeView';
  Self.TreeView.SelectedImageIndex := -1;
  Self.TreeView.Size := System.Drawing.Size.Create(92, 150);
  Self.TreeView.TabIndex := 2;
  Include(Self.TreeView.AfterSelect, Self.TreeView_AfterSelect);
  Include(Self.TreeView.BeforeExpand, Self.TreeView_BeforeExpand);
  // 
  // ExplorerViewer
  // 
  Self.Controls.Add(Self.TreeView);
  Self.Controls.Add(Self.Splitter);
  Self.Controls.Add(Self.ListView);
  Self.Name := 'ExplorerViewer';
  Self.Size := System.Drawing.Size.Create(216, 150);
  Self.ResumeLayout(False);
end;
{$ENDREGION}

constructor ExplorerViewer.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  FAutoActivate := True;
  ListView.Dock := DockStyle.Right;
  Splitter.Dock := DockStyle.Right;
  TreeView.Dock := DockStyle.Fill;
  FIconList := ArrayList.Create;
  FillTreeView;
end;

procedure ExplorerViewer.ListView_ItemActivate(sender: System.Object;
  e: System.EventArgs);
var
  I: Integer;
  FileName: string;
  Args: ExplorerViewerEventArgs;
begin
  for I := 0 to ListView.SelectedItems.Count - 1 do
  begin
    if TreeView.SelectedNode <> nil then
      FileName := IncludeBackslash(TreeView.SelectedNode.FullPath)
    else
      FileName := '';
    FileName := FileName + ListView.SelectedItems[I].Text;
    if FAutoActivate then
      ActivateFile(FileName);
    Args := ExplorerViewerEventArgs.Create;
    Args.ActiveFile := FileName;
    OnFileActivated(Args);
  end;
end;

procedure ExplorerViewer.TreeView_BeforeExpand(sender: System.Object;
  e: System.Windows.Forms.TreeViewCancelEventArgs);
begin
  if e.Node.Tag = nil then
    RefreshNode(e.Node);
end;

procedure ExplorerViewer.TreeView_AfterSelect(sender: System.Object;
  e: System.Windows.Forms.TreeViewEventArgs);
begin
  FillListView(e.Node.FullPath);
end;

procedure ExplorerViewer.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    DisposeIcons;
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

function ExplorerViewer.GetCurrentItem: string;
var
  Selected: TreeNode;
begin
  Selected := TreeView.SelectedNode;
  if Selected <> nil then
    Result := Selected.FullPath
  else
    Result := '';
end;

procedure ExplorerViewer.SetCurrentItem(const Value: string);
var
  I, J: Integer;
  Directories: array of string;
  Node: TreeNode;
  Nodes: TreeNodeCollection;
begin
  if Value <> '' then
  begin
    Node := nil;
    Nodes := TreeView.Nodes;
    // Split path string at backslashes to get all subdirs
    Directories := Value.Split(['\']);
    // Traverse tree, taking one subdir at a time
    for I := 0 to Length(Directories) - 1 do
    begin
      for J := 0 to Nodes.Count - 1 do
      begin
        if System.String.Compare(Nodes[J].Text, Directories[I], True) = 0
        then
        begin
          Node := Nodes[J];
          Nodes := Node.Nodes;
          Break;
        end;
      end;
    end;
    // select inner most node found
    if Node <> nil then
    begin
      TreeView.SelectedNode := Node;
      Node.Expand;
    end;
  end;
end;

procedure ExplorerViewer.FillListView(APath: string);
var
  DI: DirectoryInfo;
  FI: array of FileInfo;
  I: Integer;
  FileIcon: System.Drawing.Icon;
  Item: ListViewItem;
begin
  ListView.Items.Clear;
  ImageList.Images.Clear;
  DisposeIcons;
  DI := DirectoryInfo.Create(IncludeBackslash(APath));
  // Get array representing the files in the current directory.
  FI := DI.GetFiles;
  // add each file to the ListView
  for I := 0 to Length(FI) - 1 do
  begin
    Item := ListView.Items.Add(FI[I].Name);
    FileIcon := ExtractIcon(FI[I].FullName);
    if FileIcon <> nil then
    begin
      // Add icon to image list and save to ArrayList so it can be Disposed
      FIconList.Add(FileIcon);
      ImageList.Images.Add(FileIcon);
      Item.ImageIndex := ImageList.Images.Count - 1;
    end;
  end;
end;

procedure ExplorerViewer.FillTreeView;
var
  LogDrives: array of string;
  I: Integer;
begin
  // Root nodes of tree are all logical drives
  TreeView.Nodes.Clear;
  DisposeIcons;
  LogDrives := Directory.GetLogicalDrives;
  for I := 0 to Length(LogDrives) - 1 do
    AddTreeNode(nil, LogDrives[I]);
end;

procedure ExplorerViewer.RefreshNode(Node: TreeNode);
var
  SubDirs: array of string;
  I: Integer;
begin
  // Node Tag indicates that directory has been processed
  Node.Tag := TObject(True);
  // Get list of subdirectories for this node
  SubDirs := Directory.GetDirectories(IncludeBackslash(Node.FullPath));
  // Prune tree at this point and add direct children
  Node.Nodes.Clear;
  // Add each subdir to tree
  for I := 0 to Length(SubDirs) - 1 do
    AddTreeNode(Node, Path.GetFileName(SubDirs[I]));
end;

procedure ExplorerViewer.AddTreeNode(Parent: TreeNode; NodeName: string);
var
  Nodes: TreeNodeCollection;
  NewNode: TreeNode;
begin
  if Parent = nil then
    Nodes := TreeView.Nodes
  else
    Nodes := Parent.Nodes;
  NewNode := Nodes.Add(ExcludeBackslash(NodeName));
  NewNode.Nodes.Add('');  // add "fake" node to make plus sign appear
end;

function ExplorerViewer.ExcludeBackslash(APath: string): string;
begin
  if APath.EndsWith('\') then
    Result := APath.SubString(0, APath.Length - 1)
  else
    Result := APath;
end;

function ExplorerViewer.IncludeBackslash(APath: string): string;
begin
  if not APath.EndsWith('\') then
    Result := APath + '\'
  else
    Result := APath;
end;

procedure ExplorerViewer.DisposeIcons;
var
  I: Integer;
begin
  for I := 0 to FIconList.Count - 1 do
  begin
    if FIconlist[I] <> nil then
      (FIconList[I] as System.Drawing.Icon).Dispose;
  end;
  FIconList.Clear;
end;

procedure ExplorerViewer.OnFileActivated(Args: ExplorerViewerEventArgs);
begin
  if Assigned(FFileActivated) then
    FFileActivated(Self, Args);
end;

procedure ExplorerViewer.ActivateFile(FileName: string);
var
  Proc: System.Diagnostics.Process;
begin
  Proc := System.Diagnostics.Process.Create;
  try
    Proc.StartInfo.FileName := FileName;
    Proc.StartInfo.UseShellExecute := True;
    Proc.StartInfo.ErrorDialog := True;
    Proc.Start;
  finally
    Proc.Dispose;
  end;
end;

end.
