unit d8dg.SimpleStatus;

interface

uses System.Windows.Forms, System.ComponentModel, System.Collections,
  System.Windows.Forms.Design, System.ComponentModel.Design;

type
  [ProvideProperty('StatusText', 'System.Object')]
  SimpleStatusBar = class(System.Windows.Forms.StatusBar, IExtenderProvider)
  private
    FStatusTexts: HashTable;
    FActiveControl: Control;
    procedure HandleControlEnter(Sender: TObject; e: EventArgs);
    procedure HandleControlLeave(Sender: TObject; e: EventArgs);
  strict protected
    procedure Dispose(disposing: Boolean); override;
  public
    constructor Create;
    function CanExtend(extendee: TObject): Boolean;
    function GetStatusText(Ctl: TObject): string;
    procedure SetStatusText(Ctl: TObject; Value: string);
  end;

implementation

{ SimpleStatusBar }

function SimpleStatusBar.CanExtend(extendee: TObject): Boolean;
begin
  Result := (extendee is Control) and (not (extendee is SimpleStatusBar));
end;

constructor SimpleStatusBar.Create;
begin
  inherited;
  FStatusTexts := HashTable.Create;
end;

procedure SimpleStatusBar.Dispose(disposing: Boolean);
var
  I, Size: Integer;
  A: array of TObject;
begin
  if disposing then
  begin
    // make sure all event handlers are unhooked
    Size := FStatusTexts.Count;
    SetLength(A, Size);
    FStatusTexts.Keys.CopyTo(A, 0);
    for I := 0 to Size - 1 do
      SetStatusText(A[I] as Control, '');
  end;
  inherited;
end;

function SimpleStatusBar.GetStatusText(Ctl: TObject): string;
begin
  Result := string(FStatusTexts[Ctl as System.Windows.Forms.Control]);
  if Result = nil then
    Result := '';
end;

procedure SimpleStatusBar.HandleControlEnter(Sender: TObject;
  e: EventArgs);
begin
  FActiveControl := Sender as Control;
  Self.Text := GetStatusText(FActiveControl);
end;

procedure SimpleStatusBar.HandleControlLeave(Sender: TObject; e: EventArgs);
begin
  if Sender = FActiveControl then
  begin
    FActiveControl := nil;
    Self.Text := '';
  end;
end;

procedure SimpleStatusBar.SetStatusText(Ctl: TObject; Value: string);
var
  C: Control;
begin
  C := Ctl as System.Windows.Forms.Control;
  if Value = nil then
    Value := '';
  if Value.Length = 0 then
  begin
    FStatusTexts.Remove(C);
    Exclude(C.Enter, HandleControlEnter);
    Exclude(C.Leave, HandleControlLeave);
  end
  else begin
    FStatusTexts[Ctl] := Value;
    Include(C.Enter, HandleControlEnter);
    Include(C.Leave, HandleControlLeave);
  end;
  if C = FActiveControl then
    Text := Value;
end;

end.
