unit d8dg.Worthless;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Globalization;

type
  EnumType = (Zero, One, Two, Three);

type
  SomeObjectConverter = class(ExpandableObjectConverter)
  public
    function CanConvertFrom(context: ITypeDescriptorContext;
      t: System.Type): Boolean; override;
    function ConvertFrom(context: ITypeDescriptorContext; info: CultureInfo;
      value: TObject): TObject; override;
    function ConvertTo(context: ITypeDescriptorContext; culture: CultureInfo;
      value: TObject; destType: System.Type): TObject; override;
  end;

  [TypeConverter(TypeOf(SomeObjectConverter))]
  SomeObject = class(TObject)
  private
    FProp1: Integer;
    FProp2: string;
  public
    constructor Create;
  published
    property Prop1: Integer read FProp1 write FProp1;
    property Prop2: string read FProp2 write FProp2;
  end;

	[ToolboxItem(True)]
  JustWorthless = class(System.Windows.Forms.Control)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    // Internal Data Storage
    FIntegerProp: Integer;
    FStringProp: string;
    FCharProp: Char;
    FBooleanProp: Boolean;
    FEnumProp: EnumType;
    FSomeObj: SomeObject;
  protected
    procedure Init; virtual;
  public
    constructor Create; overload;
    constructor Create(Container: System.ComponentModel.IContainer); overload;
  published
    // Simple property types
    property IntegerProp: Integer read FIntegerProp write FIntegerProp default 100;
    property StringProp: string read FStringProp write FStringProp;
    property CharProp: Char read FCharProp write FCharProp;
    property BooleanProp: Boolean read FBooleanProp write FBooleanProp;
    // enum property
    property EnumProp: EnumType read FEnumProp write FEnumProp;
    // object property
    property SomeObj: SomeObject read FSomeObj write FSomeObj;
  end;

implementation

{ SomeObjectConverter }

function SomeObjectConverter.CanConvertFrom(context: ITypeDescriptorContext;
  t: System.Type): Boolean;
begin
  if t = TypeOf(System.String) then
    Result := True
  else
    Result := inherited CanConvertFrom(context, t);
end;

function SomeObjectConverter.ConvertFrom(context: ITypeDescriptorContext;
  info: CultureInfo; value: TObject): TObject;
const
  Seps: array[0..0] of char = (',');
var
  S: string;
  PropStrings: array of string;
  Obj: SomeObject;
begin
  if value is System.String then
  begin
    S := value as System.String;
    if S = '' then
      Result := SomeObject.Create
    else begin
      // parse the format "Prop1, Prop2"
      PropStrings := S.Split(Seps);
      if Length(PropStrings) <> 2 then
        raise ArgumentException.Create(System.String.Format(
          'Cannot convert "{0}" to type SomeObject', S));
      Obj := SomeObject.Create;
      Obj.Prop1 := Convert.ToInt32(PropStrings[0].Trim);
      Obj.Prop2 := PropStrings[1].Trim;
      Result := Obj;
    end;
  end
  else
    Result := inherited ConvertFrom(context, info, value);
end;

function SomeObjectConverter.ConvertTo(context: ITypeDescriptorContext;
  culture: CultureInfo; value: TObject; destType: System.Type): TObject;
var
  Obj: SomeObject;
begin
  if (destType = TypeOf(System.String)) and (value is SomeObject) then
  begin
    Obj := value as SomeObject;
    // build the string as "Prop1, Prop2"
    Result := Obj.Prop1.ToString + ', ' + Obj.Prop2;
  end
  else
    Result := inherited ConvertTo(context, culture, value, destType);
end;

{ SomeObject }

constructor SomeObject.Create;
begin
  inherited;
  FProp1 := 1971;
  FProp2 := 'hello';
end;

{ JustWorthless }

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure JustWorthless.InitializeComponent;
begin
  Self.Components := System.ComponentModel.Container.Create;
end;
{$ENDREGION}

constructor JustWorthless.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  Init;
end;

constructor JustWorthless.Create(Container: System.ComponentModel.IContainer);
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  Container.Add(Self);
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  Init;
end;

procedure JustWorthless.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

procedure JustWorthless.Init;
begin
  FSomeObj := SomeObject.Create;
  FIntegerProp := 100;
end;

end.
