program AsyncDelegate;
{$APPTYPE CONSOLE}
//
// This example demonstrates how to call a delegate asynchronously using
// BeginInvoke() and EndInvoke()
//
uses
  System.Reflection,
  System.Threading;

type
  TMyDelegate = procedure of object;

  TMyClass = class
  private 
    FOnDoSomething : TMyDelegate;
  public
    procedure CallDelegate;
    property  OnDoSomething : TMyDelegate read FOnDoSomething write FOnDoSomething;
  end;

  TAnotherClass = class
  public
    procedure DoFoo;
  end;

procedure TMyClass.CallDelegate;
var
  obj : System.Object;
  t : System.Type;
  m : MethodInfo;
  parms : array [0..1] of System.Object;
begin
  writeln('CallDelegate');
  if Assigned(FOnDoSomething) then
  begin
    // writeln('call delegate');
    // Normally this would look similar to
    // @FOnDoSomething.BeginInvoke(nil, nil);
    // but the compiler doesn't support BeginInvoke, since
    // it thinks it's just a pointer to a method.
    //
    // the work around uses reflection to invoke the method
    // first cast the "method pointer" to an object
    obj := System.Object(@FOnDoSomething);
    // now get the type of the FOnDoSomething
    t := obj.GetType;
    // now we gat search for a method named BeginInvoke
    m := t.GetMethod('BeginInvoke');
    // build the parameter list
    parms[0] := nil;
    parms[1] := nil;
    // now we can call BeginInvoke with the parms
    m.Invoke(obj, parms);
  end;
end;

procedure TAnotherClass.DoFoo;
begin
  writeln(AppDomain.GetCurrentThreadID,' DoFoo');
end;

var
  c : TMyClass;
  ac : TAnotherClass;

begin
  c := TMyClass.Create;
  ac := TAnotherClass.Create;
  writeln(AppDomain.GetCurrentThreadID,' assigning the delegate');
  c.OnDoSomething := @ac.DoFoo;
  writeln(AppDomain.GetCurrentThreadID,' calling the delegate');
  c.CallDelegate;
  // give the delegate some time to call it...
  Thread.Sleep(2000);
  writeln(AppDomain.GetCurrentThreadID,' Done');
end.
