program TestRWLock;
{$APPTYPE CONSOLE}

//
// This example demonstrates how to use the ReaderWriterLock class
//

uses
  System.Threading,
  Borland.Vcl.SysUtils,
  rwlockthreads in 'rwlockthreads.pas';

const
  NUM_READ_THREADS = 5;

var
  i           : integer;
  rdthrds     : array of TReadThread;
  writethread : TWriteThread;
  rwlck       : System.Threading.ReaderWriterLock;

begin
  writeln('Starting TestRWLock..');
  SetLength(rdthrds,NUM_READ_THREADS);
  try
    // create the RW Lock
    rwlck := System.Threading.ReaderWriterLock.Create;
    try
      try
        // create the read threads, suspended
        writeln('Creating read threads..');
        for i:=0 to NUM_READ_THREADS-1 do
        begin
          rdthrds[i] := TReadThread.Create(true);
          rdthrds[i].ReadWriteLock := rwlck;
          rdthrds[i].FreeOnTerminate := true;
          rdthrds[i].Resume;
        end;

        writeln('Creating write thread');
        // create the write thread, suspended
        writethread := TWriteThread.Create(true);
        try
          writethread.ReadWriteLock := rwlck;
          writethread.NumberToAdd := 10;
          writethread.FreeOnTerminate := false;
          writethread.Resume;

          // now wait for the write thread to be finished
          writeln('Main Thread.. waiting for the writer..');
          writethread.WaitFor;
          writeln('Main Thread.. done waiting for writer..');
        finally
          writethread.Free;
        end;

        Thread.Sleep(1000 * 3);

        // terminate the read threads
        for i:=0 to NUM_READ_THREADS-1 do
        begin
          writeln('Terminating thread ',i);
          rdthrds[i].Terminate;
        end;

        writeln('main thread is sleeping..');
        // give some time for the termination..
        Thread.Sleep(1000 * 10);
        writeln('Everything should be done now..');
      finally
        writeln('Releasing the rwlock..');
        rwlck.Free;
      end;
    except
      on e : exception do
      begin
        writeln('Main thread exception fired ',e.message);
      end;
    end;
  finally
    writeln('Releasing the thread array..');
    SetLength(rdthrds,0);
  end;
  writeln('Done with TestRWlock');
end.
