unit WinForm;

//
// This comprehensive example demonstrates how to write a WinForm multi-threaded
// application.
//   1) Creating threads of all types
//        Instance, Class, Timer, ThreadPool, Begin/EndInvoke
//   2) Controlling threads
//         Abort, Interrupt, Suspend/Resume, Join
//   3) How to safely update controls from a thread
//   4) Waiting and Signaling other threads
//   5) Using GDI+ with multiple threads
//

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Data, System.Threading;

type
  TWinForm = class(System.Windows.Forms.Form)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    GroupBox1: System.Windows.Forms.GroupBox;
    rdBtnClass: System.Windows.Forms.RadioButton;
    rdBtnInstance: System.Windows.Forms.RadioButton;
    rdBtnThreadPool: System.Windows.Forms.RadioButton;
    rdBtnTimer: System.Windows.Forms.RadioButton;
    progBar: System.Windows.Forms.ProgressBar;
    trackBar: System.Windows.Forms.TrackBar;
    Panel1: System.Windows.Forms.Panel;
    btnCreateThread: System.Windows.Forms.Button;
    btnCreateGraphics: System.Windows.Forms.Button;
    btnResetControls: System.Windows.Forms.Button;
    textBox: System.Windows.Forms.TextBox;
    Label1: System.Windows.Forms.Label;
    txtBoxStartAt: System.Windows.Forms.TextBox;
    Label2: System.Windows.Forms.Label;
    GroupBox2: System.Windows.Forms.GroupBox;
    rdBtnAbort: System.Windows.Forms.RadioButton;
    rdBtnInterrupt: System.Windows.Forms.RadioButton;
    rdBtnSuspendResume: System.Windows.Forms.RadioButton;
    rdBtnJoin: System.Windows.Forms.RadioButton;
    btnControlThread: System.Windows.Forms.Button;
    btnKillTimerThread: System.Windows.Forms.Button;
    rdbtnUI: System.Windows.Forms.RadioButton;
    chkbxDoEvents: System.Windows.Forms.CheckBox;
    GroupBox3: System.Windows.Forms.GroupBox;
    rdBtnBeginEndInvoke: System.Windows.Forms.RadioButton;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
    procedure btnCreateThread_Click(sender: System.Object; e: System.EventArgs);
    procedure TWinForm_Load(sender: System.Object; e: System.EventArgs);
    procedure btnControlThread_Click(sender: System.Object; e: System.EventArgs);
    procedure btnCreateGraphics_Click(sender: System.Object; e: System.EventArgs);
    procedure btnKillTimerThread_Click(sender: System.Object; e: System.EventArgs);
    procedure TWinForm_Closing(sender: System.Object; e: System.ComponentModel.CancelEventArgs);
    procedure ThreadType_OnClick(sender: System.Object; e: System.EventArgs);
    procedure btnResetControls_Click(sender: System.Object; e: System.EventArgs);
  private
    FStartNum: integer;
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  protected
    FCurrentThread : Thread;
    FWaitingThread : Thread;
    FTimer         : Timer;
    bUsingUIThread : boolean;
  public
    constructor Create;
    procedure UpdateControls;
    procedure UpdateControlsNoWaiting;
    procedure LongRunningMethod(msg : System.Object);
    procedure UpdateTextBox(msg : System.Object);
    procedure EnableCreateThreadButton;

    procedure InstanceThreadMe;
    procedure PoolThreadMe(state : System.Object);
    procedure TimerThreadMe(state : System.Object);
    property StartingNumber : integer read FStartNum write FStartNum;
  end;

implementation

{$R 'WinForm.TWinForm.resources'}

type
  TMyUpdateDelegate = procedure of object;
  TMyTextBoxDelegate = procedure(msg : System.Object) of object;

  TMyClass = class
  public
    class procedure ThreadMe; static;
    class procedure WaitForThreadToFinish; static;
    class procedure BeginEndInvokeThread; static;
  end;

  TMyGraphicsClass = class
  strict private
    FControl : System.Windows.Forms.Control;
    FDrawRectangle : boolean;
  public
    procedure Draw;
    property Control : System.Windows.Forms.Control write FControl;
    property DrawRectangle : boolean write FDrawRectangle;
  end;

var
  theWinFormInstance : TWinForm;
  threadIsFinished : System.Threading.AutoResetEvent;

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TWinForm.InitializeComponent;
begin
  Self.GroupBox1 := System.Windows.Forms.GroupBox.Create;
  Self.rdBtnBeginEndInvoke := System.Windows.Forms.RadioButton.Create;
  Self.Label2 := System.Windows.Forms.Label.Create;
  Self.txtBoxStartAt := System.Windows.Forms.TextBox.Create;
  Self.btnKillTimerThread := System.Windows.Forms.Button.Create;
  Self.chkbxDoEvents := System.Windows.Forms.CheckBox.Create;
  Self.rdbtnUI := System.Windows.Forms.RadioButton.Create;
  Self.btnCreateThread := System.Windows.Forms.Button.Create;
  Self.rdBtnTimer := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnThreadPool := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnInstance := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnClass := System.Windows.Forms.RadioButton.Create;
  Self.Panel1 := System.Windows.Forms.Panel.Create;
  Self.btnCreateGraphics := System.Windows.Forms.Button.Create;
  Self.textBox := System.Windows.Forms.TextBox.Create;
  Self.Label1 := System.Windows.Forms.Label.Create;
  Self.GroupBox2 := System.Windows.Forms.GroupBox.Create;
  Self.btnControlThread := System.Windows.Forms.Button.Create;
  Self.rdBtnJoin := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnSuspendResume := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnInterrupt := System.Windows.Forms.RadioButton.Create;
  Self.rdBtnAbort := System.Windows.Forms.RadioButton.Create;
  Self.GroupBox3 := System.Windows.Forms.GroupBox.Create;
  Self.trackBar := System.Windows.Forms.TrackBar.Create;
  Self.progBar := System.Windows.Forms.ProgressBar.Create;
  Self.btnResetControls := System.Windows.Forms.Button.Create;
  Self.GroupBox1.SuspendLayout;
  Self.GroupBox2.SuspendLayout;
  Self.GroupBox3.SuspendLayout;
  (System.ComponentModel.ISupportInitialize(Self.trackBar)).BeginInit;
  Self.SuspendLayout;
  // 
  // GroupBox1
  // 
  Self.GroupBox1.Controls.Add(Self.rdBtnBeginEndInvoke);
  Self.GroupBox1.Controls.Add(Self.Label2);
  Self.GroupBox1.Controls.Add(Self.txtBoxStartAt);
  Self.GroupBox1.Controls.Add(Self.btnKillTimerThread);
  Self.GroupBox1.Controls.Add(Self.chkbxDoEvents);
  Self.GroupBox1.Controls.Add(Self.rdbtnUI);
  Self.GroupBox1.Controls.Add(Self.btnCreateThread);
  Self.GroupBox1.Controls.Add(Self.rdBtnTimer);
  Self.GroupBox1.Controls.Add(Self.rdBtnThreadPool);
  Self.GroupBox1.Controls.Add(Self.rdBtnInstance);
  Self.GroupBox1.Controls.Add(Self.rdBtnClass);
  Self.GroupBox1.Location := System.Drawing.Point.Create(8, 9);
  Self.GroupBox1.Name := 'GroupBox1';
  Self.GroupBox1.Size := System.Drawing.Size.Create(208, 183);
  Self.GroupBox1.TabIndex := 0;
  Self.GroupBox1.TabStop := False;
  Self.GroupBox1.Text := 'Thread Types';
  // 
  // rdBtnBeginEndInvoke
  // 
  Self.rdBtnBeginEndInvoke.Location := System.Drawing.Point.Create(16, 120);
  Self.rdBtnBeginEndInvoke.Name := 'rdBtnBeginEndInvoke';
  Self.rdBtnBeginEndInvoke.Size := System.Drawing.Size.Create(112, 24);
  Self.rdBtnBeginEndInvoke.TabIndex := 16;
  Self.rdBtnBeginEndInvoke.Text := 'Begin/End Invoke';
  Include(Self.rdBtnBeginEndInvoke.Click, Self.ThreadType_OnClick);
  // 
  // Label2
  // 
  Self.Label2.Location := System.Drawing.Point.Create(88, 36);
  Self.Label2.Name := 'Label2';
  Self.Label2.Size := System.Drawing.Size.Create(48, 16);
  Self.Label2.TabIndex := 15;
  Self.Label2.Text := 'Start at:';
  // 
  // txtBoxStartAt
  // 
  Self.txtBoxStartAt.Enabled := False;
  Self.txtBoxStartAt.Location := System.Drawing.Point.Create(144, 34);
  Self.txtBoxStartAt.Name := 'txtBoxStartAt';
  Self.txtBoxStartAt.Size := System.Drawing.Size.Create(40, 20);
  Self.txtBoxStartAt.TabIndex := 14;
  Self.txtBoxStartAt.Text := '17';
  Self.txtBoxStartAt.TextAlign := System.Windows.Forms.HorizontalAlignment.Right;
  // 
  // btnKillTimerThread
  // 
  Self.btnKillTimerThread.Enabled := False;
  Self.btnKillTimerThread.Location := System.Drawing.Point.Create(112, 97);
  Self.btnKillTimerThread.Name := 'btnKillTimerThread';
  Self.btnKillTimerThread.TabIndex := 13;
  Self.btnKillTimerThread.Text := 'Kill Timer';
  Include(Self.btnKillTimerThread.Click, Self.btnKillTimerThread_Click);
  // 
  // chkbxDoEvents
  // 
  Self.chkbxDoEvents.Location := System.Drawing.Point.Create(96, 17);
  Self.chkbxDoEvents.Name := 'chkbxDoEvents';
  Self.chkbxDoEvents.Size := System.Drawing.Size.Create(104, 16);
  Self.chkbxDoEvents.TabIndex := 12;
  Self.chkbxDoEvents.Text := 'Call DoEvents()';
  // 
  // rdbtnUI
  // 
  Self.rdbtnUI.Checked := True;
  Self.rdbtnUI.Location := System.Drawing.Point.Create(16, 17);
  Self.rdbtnUI.Name := 'rdbtnUI';
  Self.rdbtnUI.Size := System.Drawing.Size.Create(64, 16);
  Self.rdbtnUI.TabIndex := 0;
  Self.rdbtnUI.TabStop := True;
  Self.rdbtnUI.Text := 'UI';
  Include(Self.rdbtnUI.Click, Self.ThreadType_OnClick);
  // 
  // btnCreateThread
  // 
  Self.btnCreateThread.Location := System.Drawing.Point.Create(8, 151);
  Self.btnCreateThread.Name := 'btnCreateThread';
  Self.btnCreateThread.Size := System.Drawing.Size.Create(184, 23);
  Self.btnCreateThread.TabIndex := 5;
  Self.btnCreateThread.Text := 'Create Thread';
  Include(Self.btnCreateThread.Click, Self.btnCreateThread_Click);
  // 
  // rdBtnTimer
  // 
  Self.rdBtnTimer.Location := System.Drawing.Point.Create(16, 97);
  Self.rdBtnTimer.Name := 'rdBtnTimer';
  Self.rdBtnTimer.Size := System.Drawing.Size.Create(56, 24);
  Self.rdBtnTimer.TabIndex := 4;
  Self.rdBtnTimer.Text := 'Timer';
  Include(Self.rdBtnTimer.Click, Self.ThreadType_OnClick);
  // 
  // rdBtnThreadPool
  // 
  Self.rdBtnThreadPool.Location := System.Drawing.Point.Create(16, 80);
  Self.rdBtnThreadPool.Name := 'rdBtnThreadPool';
  Self.rdBtnThreadPool.Size := System.Drawing.Size.Create(80, 16);
  Self.rdBtnThreadPool.TabIndex := 3;
  Self.rdBtnThreadPool.Text := 'Threadpool';
  Include(Self.rdBtnThreadPool.Click, Self.ThreadType_OnClick);
  // 
  // rdBtnInstance
  // 
  Self.rdBtnInstance.Location := System.Drawing.Point.Create(16, 32);
  Self.rdBtnInstance.Name := 'rdBtnInstance';
  Self.rdBtnInstance.Size := System.Drawing.Size.Create(72, 24);
  Self.rdBtnInstance.TabIndex := 1;
  Self.rdBtnInstance.Text := 'Instance';
  Include(Self.rdBtnInstance.Click, Self.ThreadType_OnClick);
  // 
  // rdBtnClass
  // 
  Self.rdBtnClass.Location := System.Drawing.Point.Create(16, 54);
  Self.rdBtnClass.Name := 'rdBtnClass';
  Self.rdBtnClass.Size := System.Drawing.Size.Create(56, 24);
  Self.rdBtnClass.TabIndex := 2;
  Self.rdBtnClass.Text := 'Class';
  Include(Self.rdBtnClass.Click, Self.ThreadType_OnClick);
  // 
  // Panel1
  // 
  Self.Panel1.Location := System.Drawing.Point.Create(8, 240);
  Self.Panel1.Name := 'Panel1';
  Self.Panel1.Size := System.Drawing.Size.Create(320, 248);
  Self.Panel1.TabIndex := 4;
  // 
  // btnCreateGraphics
  // 
  Self.btnCreateGraphics.Location := System.Drawing.Point.Create(8, 200);
  Self.btnCreateGraphics.Name := 'btnCreateGraphics';
  Self.btnCreateGraphics.Size := System.Drawing.Size.Create(320, 32);
  Self.btnCreateGraphics.TabIndex := 3;
  Self.btnCreateGraphics.Text := 'Multiple threads with GDI+ and CreateGraphics()';
  Include(Self.btnCreateGraphics.Click, Self.btnCreateGraphics_Click);
  // 
  // textBox
  // 
  Self.textBox.Location := System.Drawing.Point.Create(336, 224);
  Self.textBox.Multiline := True;
  Self.textBox.Name := 'textBox';
  Self.textBox.ScrollBars := System.Windows.Forms.ScrollBars.Both;
  Self.textBox.Size := System.Drawing.Size.Create(320, 264);
  Self.textBox.TabIndex := 8;
  Self.textBox.Text := '';
  // 
  // Label1
  // 
  Self.Label1.Location := System.Drawing.Point.Create(336, 204);
  Self.Label1.Name := 'Label1';
  Self.Label1.Size := System.Drawing.Size.Create(200, 16);
  Self.Label1.TabIndex := 8;
  Self.Label1.Text := 'Messages / Debug Information';
  // 
  // GroupBox2
  // 
  Self.GroupBox2.Controls.Add(Self.btnControlThread);
  Self.GroupBox2.Controls.Add(Self.rdBtnJoin);
  Self.GroupBox2.Controls.Add(Self.rdBtnSuspendResume);
  Self.GroupBox2.Controls.Add(Self.rdBtnInterrupt);
  Self.GroupBox2.Controls.Add(Self.rdBtnAbort);
  Self.GroupBox2.Location := System.Drawing.Point.Create(464, 8);
  Self.GroupBox2.Name := 'GroupBox2';
  Self.GroupBox2.Size := System.Drawing.Size.Create(184, 184);
  Self.GroupBox2.TabIndex := 2;
  Self.GroupBox2.TabStop := False;
  Self.GroupBox2.Text := 'Control Class && Instance Thread';
  // 
  // btnControlThread
  // 
  Self.btnControlThread.Enabled := False;
  Self.btnControlThread.Location := System.Drawing.Point.Create(24, 152);
  Self.btnControlThread.Name := 'btnControlThread';
  Self.btnControlThread.Size := System.Drawing.Size.Create(104, 23);
  Self.btnControlThread.TabIndex := 4;
  Self.btnControlThread.Text := 'Control Thread';
  Include(Self.btnControlThread.Click, Self.btnControlThread_Click);
  // 
  // rdBtnJoin
  // 
  Self.rdBtnJoin.Location := System.Drawing.Point.Create(16, 120);
  Self.rdBtnJoin.Name := 'rdBtnJoin';
  Self.rdBtnJoin.TabIndex := 3;
  Self.rdBtnJoin.Text := 'Join (Wait)';
  // 
  // rdBtnSuspendResume
  // 
  Self.rdBtnSuspendResume.Location := System.Drawing.Point.Create(16, 88);
  Self.rdBtnSuspendResume.Name := 'rdBtnSuspendResume';
  Self.rdBtnSuspendResume.Size := System.Drawing.Size.Create(128, 24);
  Self.rdBtnSuspendResume.TabIndex := 2;
  Self.rdBtnSuspendResume.Text := 'Suspend / Resume';
  // 
  // rdBtnInterrupt
  // 
  Self.rdBtnInterrupt.Location := System.Drawing.Point.Create(16, 56);
  Self.rdBtnInterrupt.Name := 'rdBtnInterrupt';
  Self.rdBtnInterrupt.TabIndex := 1;
  Self.rdBtnInterrupt.Text := 'Interrupt';
  // 
  // rdBtnAbort
  // 
  Self.rdBtnAbort.Checked := True;
  Self.rdBtnAbort.Location := System.Drawing.Point.Create(16, 24);
  Self.rdBtnAbort.Name := 'rdBtnAbort';
  Self.rdBtnAbort.TabIndex := 0;
  Self.rdBtnAbort.TabStop := True;
  Self.rdBtnAbort.Text := 'Abort';
  // 
  // GroupBox3
  // 
  Self.GroupBox3.Controls.Add(Self.trackBar);
  Self.GroupBox3.Controls.Add(Self.progBar);
  Self.GroupBox3.Controls.Add(Self.btnResetControls);
  Self.GroupBox3.Location := System.Drawing.Point.Create(224, 8);
  Self.GroupBox3.Name := 'GroupBox3';
  Self.GroupBox3.Size := System.Drawing.Size.Create(232, 184);
  Self.GroupBox3.TabIndex := 9;
  Self.GroupBox3.TabStop := False;
  Self.GroupBox3.Text := 'Controls';
  // 
  // trackBar
  // 
  Self.trackBar.LargeChange := 20;
  Self.trackBar.Location := System.Drawing.Point.Create(8, 128);
  Self.trackBar.Maximum := 100;
  Self.trackBar.Name := 'trackBar';
  Self.trackBar.Size := System.Drawing.Size.Create(208, 42);
  Self.trackBar.TabIndex := 10;
  Self.trackBar.TickFrequency := 5;
  // 
  // progBar
  // 
  Self.progBar.Location := System.Drawing.Point.Create(8, 88);
  Self.progBar.Name := 'progBar';
  Self.progBar.Size := System.Drawing.Size.Create(208, 16);
  Self.progBar.TabIndex := 9;
  // 
  // btnResetControls
  // 
  Self.btnResetControls.Location := System.Drawing.Point.Create(8, 24);
  Self.btnResetControls.Name := 'btnResetControls';
  Self.btnResetControls.Size := System.Drawing.Size.Create(208, 32);
  Self.btnResetControls.TabIndex := 8;
  Self.btnResetControls.Text := 'Reset Controls';
  Include(Self.btnResetControls.Click, Self.btnResetControls_Click);
  // 
  // TWinForm
  // 
  Self.AutoScaleBaseSize := System.Drawing.Size.Create(5, 13);
  Self.ClientSize := System.Drawing.Size.Create(664, 493);
  Self.Controls.Add(Self.GroupBox3);
  Self.Controls.Add(Self.GroupBox2);
  Self.Controls.Add(Self.textBox);
  Self.Controls.Add(Self.Label1);
  Self.Controls.Add(Self.btnCreateGraphics);
  Self.Controls.Add(Self.Panel1);
  Self.Controls.Add(Self.GroupBox1);
  Self.Name := 'TWinForm';
  Self.StartPosition := System.Windows.Forms.FormStartPosition.CenterParent;
  Self.Text := 'WinForm Threading Examples';
  Include(Self.Closing, Self.TWinForm_Closing);
  Include(Self.Load, Self.TWinForm_Load);
  Self.GroupBox1.ResumeLayout(False);
  Self.GroupBox2.ResumeLayout(False);
  Self.GroupBox3.ResumeLayout(False);
  (System.ComponentModel.ISupportInitialize(Self.trackBar)).EndInit;
  Self.ResumeLayout(False);
end;
{$ENDREGION}

procedure TWinForm.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose;
    FWaitingThread.Abort;
    if FTimer <> nil then
      FTimer.Dispose;
  end;
  inherited Dispose(Disposing);
end;

constructor TWinForm.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  theWinFormInstance := Self;
  // create an event that is used to know when a thread is finished
  threadIsFinished := System.Threading.AutoResetEvent.Create(false);
  // create a waiting thread that all it does is wait for a thread
  // to finish and update the UI thread so it can re-enable/disable
  // some buttons as appropriate
  FWaitingThread   := Thread.Create(@TMyClass.WaitForThreadToFinish);
  FWaitingThread.Start;
end;

procedure TWinForm.UpdateControls;
var
  myDelegate : TMyUpdateDelegate;
  tmpStr     : string;
begin
  // used by either the UI or worker threads, UpdateControls
  // determines if it needs to call Invoke() or just manipulate the
  // controls directly.
  if progBar.InvokeRequired then
  begin
    tmpStr := System.String.Format('{0} invoke required!',
               [AppDomain.GetCurrentThreadID]);
    UpdateTextBox(System.Object(tmpStr));
    myDelegate := @Self.UpdateControls;
    progBar.Invoke(System.Delegate(@myDelegate));
  end
  else
  begin
    progBar.Increment(1);
    trackBar.Value := trackBar.Value + 1;
  end;
end;

procedure TWinForm.UpdateControlsNoWaiting;
var
  myDelegate : TMyTextBoxDelegate;
  tmpStr     : string;
  parms      : array [0..0] of System.Object;
  lastAsyncInvoke : IAsyncResult;
begin
  //
  // This method demonstrates using BeginInvoke() and EndInvoke() in order to
  // execute a delegate asynchronously.
  //
  if textBox.InvokeRequired then
  begin
   tmpStr := System.String.Format('{0} Invoke required in UpdateControlsNoWaiting',
               [AppDomain.GetCurrentThreadID]);
   myDelegate := @Self.LongRunningMethod;
   parms[0] := System.Object(tmpStr);
   lastAsyncInvoke := textBox.BeginInvoke(System.Delegate(@myDelegate), parms);
   tmpStr := System.String.Format('{0} After BeginInvoke call in UpdateControlsNoWaiting',
               System.Object(AppDomain.GetCurrentThreadID));
   UpdateTextBox(System.Object(tmpStr));
   // now let's wait for the async call to finish..
   textBox.EndInvoke(lastAsyncInvoke);
   tmpStr := System.String.Format('{0} After EndInvoke call in UpdateControlsNoWaiting',
               [AppDomain.GetCurrentThreadID]);
   UpdateTextBox(System.Object(tmpStr));
  end
  else
    UpdateTextBox('Invoke is not required in UpdateControlsNoWaiting');
end;

procedure TWinForm.btnResetControls_Click(sender: System.Object; e: System.EventArgs);
var
  tmpStr : string;
begin
  // set the controls back to zero
  tmpStr := System.String.Format('{0} Resetting controls',
             System.Object(AppDomain.GetCurrentThreadID));
  UpdateTextBox(System.Object(tmpStr));
  progBar.Value := 0;
  trackBar.Value := 0;
end;

procedure TWinForm.TWinForm_Load(sender: System.Object; e: System.EventArgs);
var
  tmpStr : string;

begin
  // record the main UI/thread in the textbox
  textBox.Clear;
  tmpStr := System.String.Format('{0} Main/UI thread',
             System.Object(AppDomain.GetCurrentThreadID));
  UpdateTextBox(System.Object(tmpStr));
end;

procedure TWinForm.UpdateTextBox(msg : System.Object);
var
  args : array of System.Object;
  myDelegate : TMyTextBoxDelegate;

begin
  // helper method used to update the textbox from either the UI thread or a
  // worker thread.
  if textBox.InvokeRequired then
  begin
    SetLength(args,1);
    try
      myDelegate := @Self.UpdateTextBox;
      args[0] := msg;
      textBox.Invoke(System.Delegate(@myDelegate), args);
    finally
      SetLength(args,0);
    end;
  end
  else
  begin
    if textBox.Text = '' then
      textBox.Text := msg.ToString
    else
      textBox.Text := textBox.Text + Environment.NewLine + msg.ToString;
  end;
end;

//
// InstanceThreadMe is the method called from either the main ui thread or
// on an another thread. A few lines are
//

procedure TWinForm.InstanceThreadMe;
var
  tmpStr : string;
  rnd : System.Random;
  i : integer;
  thrdID : System.Object;
  whatThread : string;
begin
  try
    whatThread := 'Instance';
    if bUsingUIThread then
      whatThread := 'User Interface';

    thrdID := System.Object(AppDomain.GetCurrentThreadId);
    tmpStr := System.String.Format('{0} {1} Thread Started',thrdId, whatThread);
    UpdateTextBox(System.Object(tmpStr));

    rnd := System.Random.Create;

    for i:= FStartNum to 100 do
    begin
      if bUsingUIThread and chkbxDoEvents.Checked then
        Application.DoEvents;
      Thread.Sleep(3 * rnd.Next(500));
      UpdateControls;
    end;

    tmpStr := System.String.Format('{0} {1} Thread Finished',thrdID, whatThread);
    UpdateTextBox(System.Object(tmpStr));
  except
    on e : System.Threading.ThreadAbortException do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} {1} Thread Caught Abort Exception {2}',
                                      thrdID, whatThread, System.Object(e.message));
      UpdateTextBox(System.Object(tmpStr));
    end;
    on e : exception do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} {1} Thread Exception {2}',
                                      thrdID, whatThread, System.Object(e.message));
      UpdateTextBox(System.Object(tmpStr));
    end;
  end;

  // send an event to enable the create button
  threadIsFinished.&Set;
end;

procedure TWinForm.btnControlThread_Click(sender: System.Object; e: System.EventArgs);
var
  bReset : boolean;

begin
  // handles the controlling of the current thread.
  // not all threads can be controlled.

  // bail if we don't have a thread to deal with...
  if FCurrentThread = nil then
     Exit;

  bReset := false;

  if rdBtnAbort.Checked then
  begin
    FCurrentThread.Abort;
    bReset := true;
  end
  else if rdBtnInterrupt.Checked then
  begin
    FCurrentThread.Interrupt;
    bReset := true;
  end
  else if rdBtnSuspendResume.Checked then
  begin
    // Suspending a thread can be tricky since the thread *MUST* be
    // in the running state. Any other state will raise an exception!
    if FCurrentThread.ThreadState = System.Threading.ThreadState.Suspended then
      FCurrentThread.Resume
    else if FCurrentThread.ThreadState = System.Threading.ThreadState.Running then
      FCurrentThread.Suspend
    else
    begin
      raise Exception.Create(System.String.Format('FYI..Cannot suspend/resume a thread in {0} state!',
         System.Object(FCurrentThread.ThreadState)));
    end;
  end
  else if rdBtnJoin.Checked then
  begin
    // Joining a thread that is using Invoke is a really bad idea.
    // A deadlock will occur and the application will hang.
    // You have been warned.
    FCurrentThread.Join;
  end;

  if bReset then
  begin
    btnCreateThread.Enabled := true;
    btnResetControls.PerformClick;
  end;
end;

procedure TWinForm.btnCreateThread_Click(sender: System.Object; e: System.EventArgs);

  function UseCurrentThread : Thread;
  begin
    bUsingUIThread := true;
    // set the starting number to one
    FStartNum := 1;
    InstanceThreadMe;
    // reset FStartNum to the value of the text box
    FStartNum := System.Int32.Parse(txtBoxStartAt.Text);
    Result := System.Threading.Thread.CurrentThread;
  end;

  function CreateInstanceThread : Thread;
  var
    startingValue : integer;

  begin
    startingValue := System.Int32.Parse(txtBoxStartAt.Text);
    progBar.Value   := startingValue;
    trackBar.Value  := startingValue;
    Result := Thread.Create(@Self.InstanceThreadMe);
    Self.StartingNumber := startingValue;
    Result.Start;
    btnControlThread.Enabled := true;
    btnCreateThread.Enabled := false;
  end;

  function CreateClassThread : Thread;
  begin
    Result := Thread.Create(@TMyClass.ThreadMe);
    Result.Start;
    btnControlThread.Enabled := true;
    btnCreateThread.Enabled := false;
  end;

  function CreateThreadPoolThread : Thread;
  begin
    System.Threading.ThreadPool.QueueUserWorkItem(@Self.PoolThreadMe, nil);
    Result := nil;
    btnControlThread.Enabled := false;
    btnCreateThread.Enabled := false;
  end;

  function CreateTimerThread : Timer;
  begin
    Result := Timer.Create(@Self.TimerThreadMe, nil, 1000, 1000);
    btnControlThread.Enabled := false;
    btnKillTimerThread.Enabled := true;
  end;

  function CreateBeginEndInvokeThread : Thread;
  begin
    Result := Thread.Create(@TMyClass.BeginEndInvokeThread);
    Result.Start;
    btnControlThread.Enabled := true;
    btnCreateThread.Enabled := false;
  end;

begin
  // handles "creating" the various types of threads
  FCurrentThread := nil;
  FTimer         := nil;
  bUsingUIThread := false;
  // determine what thread to create
  if rdbtnUI.Checked then
    FCurrentThread := UseCurrentThread
  else if rdBtnInstance.Checked then
    FCurrentThread := CreateInstanceThread
  else if rdBtnClass.Checked then
    FCurrentThread := CreateClassThread
  else if rdBtnThreadPool.Checked then
    FCurrentThread := CreateThreadPoolThread
  else if rdBtnTimer.Checked then
    FTimer := CreateTimerThread
  else if rdBtnBeginEndInvoke.Checked then
    FCurrentThread := CreateBeginEndInvokeThread
  else
    raise Exception.Create('Unhandled thread type!');
end;

{ TMyClass }

class procedure TMyClass.BeginEndInvokeThread;
var
  i : integer;
  thrdID : System.Object;
  tmpStr : string;
begin
  //
  // Thread used for demonstrating BeginInvoke/EndInvoke methods
  //
  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Begin/End Invoke Thread Started',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));

  try
    for i := 1 to 5 do
    begin
      theWinFormInstance.UpdateControlsNoWaiting;
    end;
  except
    on e : exception do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} Exception in Begin/End Invoke Thread {1}',
                                      thrdID, System.Object(e.message));
      theWinFormInstance.UpdateTextBox(System.Object(tmpStr));
    end;
  end;

  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Begin/End Invoke Thread Finished',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));

  // send an event to enable the create button
  threadIsFinished.&Set;
end;

class procedure TMyClass.ThreadMe;
var
  thrdID : System.Object;
  tmpStr : string;
  rnd    : System.Random;

begin
  //
  // an example of a static class method executed on another thread
  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Static Class Thread Started',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));
  try
    rnd := System.Random.Create;
    while true do
    begin
      theWinFormInstance.UpdateControls;
      Thread.Sleep(4 * rnd.Next(500));
    end;
  except
    on e : exception do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} Exception {1}', thrdID,
                                      System.Object(e.message));
      theWinFormInstance.UpdateTextBox(System.Object(tmpStr));
    end;
  end;

  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Static Class Thread Finished',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));

  // send an event to enable the create button
  threadIsFinished.&Set;
end;


procedure TWinForm.PoolThreadMe(state : System.Object);
var
  tmpStr : string;
  rnd : System.Random;
  i : integer;
  thrdID : System.Object;
begin
  //
  // method that will be executed on a thread from the thread pool
  try
    thrdID := System.Object(AppDomain.GetCurrentThreadId);
    tmpStr := System.String.Format('{0} Pool Thread Started',thrdId);
    UpdateTextBox(System.Object(tmpStr));

    rnd := System.Random.Create;

    for i:= 1 to 100 do
    begin
      Thread.Sleep(3 * rnd.Next(500));
      UpdateControls;
    end;

    tmpStr := System.String.Format('{0} Pool Thread Finished',thrdID);
    UpdateTextBox(System.Object(tmpStr));
  except
    on e : exception do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} Pool Thread Exception {1}', thrdID,
                                       System.Object(e.message));
      UpdateTextBox(System.Object(tmpStr));
    end;
  end;
  // send an event to enable the create button
  threadIsFinished.&Set;
end;

procedure TWinForm.TimerThreadMe(state: System.Object);
var
  thrdID : System.Object;
  tmpStr : string;
begin
  // method called when the timer fires
  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Timer fired! {1}', thrdID,
                                  System.Object(DateTime.Now));
  UpdateTextBox(System.Object(tmpStr));
end;

procedure TWinForm.ThreadType_OnClick(sender: System.Object; e: System.EventArgs);
begin
  // enable/disable control helper method
  txtBoxStartAt.Enabled := rdBtnInstance.Checked;
  chkbxDoEvents.Enabled := rdbtnUI.Checked;
end;

procedure TWinForm.TWinForm_Closing(sender: System.Object; e: System.ComponentModel.CancelEventArgs);
begin
  // terminate the waiting thread
  FWaitingThread.Abort;
end;

procedure TWinForm.btnKillTimerThread_Click(sender: System.Object; e: System.EventArgs);
begin
  // demonstrates how to kill the timer thread
  if FTimer <> nil then
  begin
    FTimer.Dispose;
    FTimer := nil;
    UpdateTextBox('timer has been killed');
    btnKillTimerThread.Enabled := false;
  end;
end;

procedure TWinForm.btnCreateGraphics_Click(sender: System.Object; e: System.EventArgs);
var
  inst : array[0..1] of TMyGraphicsClass;
  thrd : Thread;
  i    : integer;
begin
  // this method creates a couple of TMyGraphicClass instances and executes
  // the draw method on different threads
  UpdateTextBox('Anything causing a repaint erase the graphics being currently drawn');
  for i:=low(inst) to high(inst) do
  begin
    inst[i] := TMyGraphicsClass.Create;
    inst[i].Control := Self.Panel1;
    inst[i].DrawRectangle := (i = 0);

    thrd := Thread.Create(@inst[i].Draw);
    thrd.Start;
  end;
end;

class procedure TMyClass.WaitForThreadToFinish;
begin
  // this method simply waits for the threadIsFinished event to
  // call the EnableCreateThreadButton
  theWinFormInstance.UpdateTextBox(System.Object('Waiting Thread started'));
  while true do
  begin
    if threadIsFinished.WaitOne then
    begin
      theWinFormInstance.UpdateTextBox(
         System.Object('Waiting Thread was signaled. Enabling create button'));
      // we can change the state of the create thread button
      theWinFormInstance.EnableCreateThreadButton;
    end;
    // now wait for another thread to finish
  end;
end;

procedure TWinForm.EnableCreateThreadButton;
var
  myDelegate : TMyUpdateDelegate;
  tmpStr     : string;
begin
  // this method determines if it is being called from a worker thread or the
  // main ui thread and enables/disables buttons appropriately.
  if btnCreateThread.InvokeRequired then
  begin
    tmpStr := System.String.Format('{0} invoke required!',
               System.Object(AppDomain.GetCurrentThreadID));
    UpdateTextBox(System.Object(tmpStr));
    myDelegate := @Self.EnableCreateThreadButton;
    btnCreateThread.Invoke(System.Delegate(@myDelegate));
  end
  else
  begin
    btnCreateThread.Enabled := true;
    btnControlThread.Enabled := false;
  end;
end;

{ TMyGraphicsClass }

procedure TMyGraphicsClass.Draw;
var
  gr : System.Drawing.Graphics;
  clr : Color;
  i   : integer;
  height : integer;
  width  : integer;
  rnd    : System.Random;
  curX   : integer;
  curY   : integer;
  curW   : integer;
  curH   : integer;
  thrdID : System.Object;
  tmpStr : string;
begin
  // this method randomly draws either Red circles or Blue Rectangles
  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Graphics Class Thread Started',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));

  try
    clr := Color.Red;
    if FDrawRectangle then
      clr := Color.Blue;

    gr := FControl.CreateGraphics;
    try
      rnd    := System.Random.Create;

      height := System.Convert.ToInt32(gr.VisibleClipBounds.Height);
      width  := System.Convert.ToInt32(gr.VisibleClipBounds.Width);

      for i:=1 to 10 do
      begin
        curX := rnd.Next(width);
        curY := rnd.Next(height);
        curW := rnd.Next(width div 3);
        curH := rnd.Next(height div 3);

        if (curX + curW) > width then
          curX := width - curW;

        if (curY + curH) > height then
          curY := height - curH;

        if FDrawRectangle then
          gr.FillRectangle(SolidBrush.Create(clr), curX, curY, curW, curH)
        else
          gr.FillEllipse(SolidBrush.Create(clr), curX, curY, curW, curH);

        Thread.Sleep(2000);
      end;
    finally
      gr.Dispose;
    end;
  except
    on e : exception do
    begin
      thrdID := System.Object(AppDomain.GetCurrentThreadId);
      tmpStr := System.String.Format('{0} Graphics class thread Exception {1}', thrdID,
                                       System.Object(e.message));
      theWinFormInstance.UpdateTextBox(System.Object(tmpStr));
    end;
  end;

  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} Graphics Class Thread Finished',thrdId);
  theWinFormInstance.UpdateTextBox(System.Object(tmpStr));

end;


procedure TWinForm.LongRunningMethod(msg : System.Object);
var
  i  : integer;
  thrdID : System.Object;
  tmpStr : string;

begin
  // simulate a long running method...
  for i := 1 to 100 do
  begin
    // DoEvents will allow updates to the UI
    Application.DoEvents;
    Thread.Sleep(100);
  end;
  // and update the text box..
  thrdID := System.Object(AppDomain.GetCurrentThreadId);
  tmpStr := System.String.Format('{0} LongRunningMethod({1})',
                                   thrdID, msg.ToString);
  textBox.Text := textBox.Text + System.Environment.NewLine + tmpStr;
end;

end.
