program ThreadingExceptions;
{$APPTYPE CONSOLE}

//
// This example demonstrates ThreadingExceptions.
//   - ThreadAbortException is handled, and reset
//   - ThreadInterruptedException is handled
//   - ThreadStateException is handled
//   - SynchronizationLockException is handled
//

uses
  System.Threading;

type
  D4DNThreadMe = class
  public
    procedure MyThreadMethod;
  end;

procedure D4DNThreadMe.MyThreadMethod;
var
  numAborts : integer;

begin
  numAborts := 0;
  while true do
  begin
    try
      write('*');
      Thread.Sleep(1000);
    except
      on e : System.Threading.ThreadInterruptedException do
      begin
        writeln('Handled ThreadInterruptedException');
      end;
      on e : System.Threading.ThreadAbortException do
      begin
        writeln('Handled threadAbortException');
        inc(numAborts);
        if numAborts = 1 then
          Thread.ResetAbort;
      end;
      on e : exception do
      begin
        writeln('Unhandled exception ',e.message);
        raise;
      end;
    end;
  end; 
end;

var
  thrdclass1 : D4DNThreadMe;
  thrd1 : Thread;
  cmd : string;
  bDone : boolean;

begin
  System.Console.WriteLine('Staring threading instance example...');

  // create myDotNetThread instance
  thrdclass1 := D4DNThreadMe.create;
  thrd1 := Thread.Create(@thrdclass1.MyThreadMethod);
  thrd1.Start;

  bDone := false;
  while not bDone do
  begin
    System.Console.WriteLine('Enter A = Abort, I = Interrupt, S = ThreadStateException, L = SyncLockException');
    cmd := System.Console.ReadLine.ToUpper;
    if (cmd = 'A') then
      thrd1.Abort // raise a ThreadAbortException
    else if (cmd = 'I') then
      thrd1.Interrupt // raise a ThreadInterruptedException
    else if (cmd = 'S') then
    begin
      try
        thrd1.Start
      except
        on e : System.Threading.ThreadStateException do
        begin
          System.Console.WriteLine('Handled ThreadStateException');
        end;
      end;
    end
    else if (cmd = 'L') then
    begin
      try
        // needs to be wrapped in an Enter/Exit() block
        System.Threading.Monitor.Wait(thrd1);
      except
        on e : System.Threading.SynchronizationLockException do
        begin
          System.Console.WriteLine('Handled SynchronizationLockException');
        end;
      end;
    end
    else
      bDone := true;
  end;

  // ensure that the thread is terminated
  thrd1.Abort;
  Thread.Sleep(1000);
  thrd1.Abort;

  System.Console.WriteLine('Done - main');
end.

