program InvMemb;
uses
  SysUtils,
  System.Reflection;
{$APPTYPE CONSOLE}

type
  TOnWriteEvent = procedure of object;

  TMyType = class(TObject)
  private
    FIntProp: Integer;
    FStrProp: String;
    FOnWriteEvent: TOnWriteEvent;
  public
    PublicStr: String;
    constructor Create(aStr: String; aInt: Integer;
      aStr2: String); override;
    procedure WriteSomething;
    procedure WriteProc;
    property IntProp: Integer read FIntProp write FIntProp;
    property StrProp: String read FStrProp write FStrProp;
    property OnWriteEvent: TOnWriteEvent read FOnWriteEvent
      write FOnWriteEvent;
  end;

constructor TMyType.Create(aStr: String; aInt: Integer; aStr2: String);
begin
  inherited Create;
  PublicStr := aStr;
  FIntProp  := aInt;
  FStrProp  := aStr2;
  OnWriteEvent := WriteProc;
end;

procedure TMyType.WriteSomething;
begin
  Console.WriteLine('PublicStr: '+PublicStr);
  Console.WriteLine('FIntProp: '+FIntProp.ToString);
  Console.WriteLine('FStrProp: '+FStrProp);
  if Assigned(FOnWriteEvent) then
    FOnWriteEvent;
end;

procedure TMyType.WriteProc;
begin
  Console.WriteLine('----In WriteProc');
end;

procedure WriteHandler;
begin
  Console.WriteLine('In WriteHandler');
end;

var
  tp:  System.Type;
  obj: System.Object;
  BFlags: BindingFlags;
  parmAray: array[0..2] of System.Object;
  s: String;
  i: Integer;
  ei: EventInfo;
  OnWrite: TOnWriteEvent;
begin
  BFlags := BindingFlags.DeclaredOnly or BindingFlags.Public
    or BindingFlags.Instance;

  tp  := TypeOf(TMyType);
  parmAray[0] := 'hello';
  parmAray[1] := System.Object(23);
  parmAray[2] := 'world';

  try
    // create an instance of the type

    obj := tp.InvokeMember('.ctor', BFlags or BindingFlags.CreateInstance,
      nil, nil,ParmAray);
    Console.WriteLine(obj.ToString);

    // Call the type's method
    (obj as TMyType).WriteSomething;
    // or
    tp.InvokeMember('WriteSomething', BFlags or BindingFlags.InvokeMethod,
      nil, obj, nil);

    // Set/Get a field
    tp.InvokeMember('PublicStr', BFlags or BindingFlags.SetField, nil, obj,
      ['67 Camaro']);

    s := String(tp.InvokeMember('PublicStr', BFlags or
      BindingFlags.GetField, nil, obj, nil));

    Console.WriteLine(s);

    // Set/Get a property
    tp.InvokeMember('IntProp', BFlags or BindingFlags.SetProperty, nil,
      obj, [23]);
    tp.InvokeMember('StrProp', BFlags or BindingFlags.SetProperty, nil,
      obj, ['Coffee']);

    i := Integer(tp.InvokeMember('IntProp', BFlags or
       BindingFlags.GetProperty, nil, obj, nil));
    s := String(tp.InvokeMember('StrProp', BFlags or
       BindingFlags.GetProperty, nil, obj, nil));

    Console.WriteLine('Get/Set Property: {0}, {1}', [i, s]);

    // Hook the event up to a handler
    ei := tp.GetEvent('OnWriteEvent');
    OnWrite := WriteHandler;
    ei.AddEventHandler(obj, System.Delegate(@OnWrite));
    (obj as TMyType).WriteSomething;

  except
    on E: Exception do
      Console.WriteLine(E.Message);
  end;

  Console.ReadLine;
end.
