program TransEx;

{$APPTYPE CONSOLE}
{%DelphiDotNetAssemblyCompiler '$(SystemRoot)\microsoft.net\framework\v1.1.4322\System.Data.dll'}

uses
  System.Data,
  System.Data.SqlClient,
  System.IO;

const
  c_cnstr = 'server=XWING;database=ddn_company;Trusted_Connection=Yes';
  c_sel_cst = 'SELECT * FROM company; SELECT * FROM contact';

function GetCommand(aFileName: String): String;
var
  sr: StreamReader;
begin
  if System.IO.File.Exists(aFileName) then
  begin
    sr := StreamReader.Create(System.IO.File.OpenRead(aFileName));
    try
      Result := sr.ReadToEnd;
    finally
      sr.Close;
    end;
  end;
end;

var
  sqlCn: SqlConnection;
  sqlDA: SqlDataAdapter;
  dsCompany: DataSet;
  sqlTrans: sqlTransaction;

  cmdInsCompany: SqlCommand;
  cmdInsContact: SqlCommand;
  drCompany: DataRow;
  drContact: DataRow;

function GetInsertContactCmd: SqlCommand;
var
  sqlCmd: SqlCommand;
begin
  sqlCmd := sqlCommand.Create(GetCommand('c_ins_cnt.sql'), sqlCn);
  sqlCmd.Parameters.Add('@first_name',  SqlDbType.NVarChar, 40,
    'first_name');
  sqlCmd.Parameters.Add('@last_name',   SqlDbType.NVarChar, 40,
    'last_name');
  sqlCmd.Parameters.Add('@mi',          SqlDbType.NVarChar, 1,
    'mi');
  sqlCmd.Parameters.Add('@department',  SqlDbType.NVarChar, 40,
    'department');
  sqlCmd.Parameters.Add('@position',    SqlDbType.NVarChar, 40,
    'position');
  sqlCmd.Parameters.Add('@work_phone',  SqlDbType.NVarChar, 20,
    'work_phone');
  sqlCmd.Parameters.Add('@home_phone',  SqlDbType.NVarChar, 20,
    'home_phone');
  sqlCmd.Parameters.Add('@fax',         SqlDbType.NVarChar, 20,
    'fax');
  sqlCmd.Parameters.Add('@cell_phone',  SqlDbType.NVarChar, 20,
    'cell_phone');
  sqlCmd.Parameters.Add('@email',       SqlDbType.NVarChar, 50,
    'email');
  sqlCmd.Parameters.Add('@company_id',  SqlDbType.Int, 0,
    'company_id');

  sqlCmd.Parameters.Add('@contact_id',  SqlDbType.Int, 0,
    'contact_id');
  sqlCmd.Parameters['@contact_id'].Direction := ParameterDirection.Output;

  sqlCmd.Parameters.Add('@row_version', SqlDbType.TimeStamp, 0,
    'row_version');
  sqlCmd.Parameters['@row_version'].Direction := ParameterDirection.Output;

  Result := sqlCmd;
end;

function GetInsertCompanyCmd: SqlCommand;
var
  sqlCmd: SqlCommand;
begin
  sqlCmd := sqlCommand.Create(GetCommand('c_ins_com.sql'), sqlCn);
  sqlCmd.Parameters.Add('@company_name',  SqlDbType.NVarChar, 100,
    'company_name');
  sqlCmd.Parameters.Add('@address_1',   SqlDbType.NVarChar, 100,
    'address_1');
  sqlCmd.Parameters.Add('@address_2',   SqlDbType.NVarChar, 100,
    'address_2');
  sqlCmd.Parameters.Add('@city',        SqlDbType.NVarChar, 40,
    'city');
  sqlCmd.Parameters.Add('@state_abbr',  SqlDbType.NVarChar, 2,
    'state_abbr');
  sqlCmd.Parameters.Add('@zip',         SqlDbType.NVarChar, 20,
    'zip');
  sqlCmd.Parameters.Add('@company_id',  SqlDbType.Int, 0,
    'company_id');
  sqlCmd.Parameters['@company_id'].Direction := ParameterDirection.Output;
  sqlCmd.Parameters.Add('@row_version', SqlDbType.TimeStamp, 0,
    'row_version');
  sqlCmd.Parameters['@row_version'].Direction := ParameterDirection.Output;

  Result := sqlCmd;
end;

function InsertCompany: DataRow;
begin
  // add a company
  Result := dsCompany.Tables['company'].NewRow;
  Result['company_name'] := 'Xapware Technologies Inc.';
  Result['address_1']    := '4164 Austin Bluffs Pkwy, #363';
  Result['city']         := 'Colorado Springs';
  Result['state_abbr']   := 'CO';
  Result['zip']          := '80918';

  // populate the SqlCommand parameters
  cmdInsCompany.Parameters['@company_name'].Value :=
    Result['company_name'];
  cmdInsCompany.Parameters['@address_1'].Value := Result['address_1'];
  cmdInsCompany.Parameters['@address_2'].Value := Result['address_2'];
  cmdInsCompany.Parameters['@city'].Value := Result['city'];
  cmdInsCompany.Parameters['@state_abbr'].Value := Result['state_abbr'];
  cmdInsCompany.Parameters['@zip'].Value := Result['zip'];
  cmdInsCompany.Transaction := sqlTrans;
  cmdInsCompany.ExecuteNonQuery;

  // assign the system generated id to the data row
  Result['company_id'] := cmdInsCompany.Parameters['@company_id'].Value;
end;

function InsertContact(acompany_id: Integer): DataRow;
begin
  Result := dsCompany.Tables['contact'].NewRow;
  Result['first_name'] := 'Xavier';
  Result['last_name']  := 'Pacheco';
  Result['mi']         := 'G';
  Result['department'] := 'Dept 1';
  Result['position']   := 'Worker bee';
  Result['work_phone'] := '123-123-1234';
  Result['home_phone'] := '234-234-2344';
  Result['fax']        := '456-456-4566';
  Result['cell_phone'] := '345-345-3654';
  Result['email']      := 'xavier@somewhere.com';
  Result['company_id'] := System.Object(acompany_id);

  // populate the SqlCommand parameters
  cmdInsContact.Parameters['@first_name'].Value := Result['first_name'];
  cmdInsContact.Parameters['@last_name'].Value := Result['last_name'];
  cmdInsContact.Parameters['@mi'].Value := Result['mi'];
  cmdInsContact.Parameters['@department'].Value := Result['department'];
  cmdInsContact.Parameters['@position'].Value := Result['position'];
  cmdInsContact.Parameters['@work_phone'].Value := Result['work_phone'];
  cmdInsContact.Parameters['@home_phone'].Value := Result['home_phone'];
  cmdInsContact.Parameters['@fax'].Value := Result['fax'];
  cmdInsContact.Parameters['@cell_phone'].Value := Result['cell_phone'];
  cmdInsContact.Parameters['@email'].Value := Result['email'];
  cmdInsContact.Parameters['@company_id'].Value :=
    System.Object(acompany_id);
  cmdInsContact.Transaction := sqlTrans;
  cmdInsContact.ExecuteNonQuery;

  // assign the system generated id to the data row
  Result['contact_id'] := cmdInsContact.Parameters['@contact_id'].Value;

end;

begin
  sqlCn := SqlConnection.Create(c_cnstr);
  sqlDA := SqlDataAdapter.Create(c_sel_cst, sqlCn);
  dsCompany := DataSet.Create('ddn_company');

  sqlDA.TableMappings.Add('Table', 'company');
  sqlDA.TableMappings.Add('Table1', 'contact');
  sqlDA.Fill(dsCompany);

  cmdInsCompany := GetInsertCompanyCmd;
  cmdInsContact := GetInsertContactCmd;

  sqlcn.Open;
  try
    sqlTrans := sqlcn.BeginTransaction;
    try
      drCompany := InsertCompany;
      drContact := InsertContact(Integer(drCompany['company_id']));
      sqlTrans.Commit;
    except
      sqlTrans.Rollback;
      raise;
    end;
    // Add the rows to the dataset
    dsCompany.Tables['company'].Rows.Add(drCompany);
    dsCompany.Tables['contact'].Rows.Add(drContact);
  finally
    sqlcn.Close;
  end;
end.
