unit BankServer_Impl;

interface

uses
  BankShared;

type
  TBankManager = class(MarshalByRefObject, IBankManager)
  private
    fAccount1,
    fAccount2 : TAccount;
  protected
    // IBankManager
    function GetAccountNumbers : TAccountNumberArray;
    function GetAccount(const AccountNumber : integer) : TAccount;
    procedure TransferMoney(const Origin, Destination : integer;
                            const Amount : double);
  public
    constructor Create;
  end;

implementation

{ TBankManager }

constructor TBankManager.Create;
begin
  inherited Create;
  fAccount1 := TAccount.Create(1, 'John Smith', 1999);
  fAccount2 := TAccount.Create(2, 'Jack Rockwell', 249);
end;

function TBankManager.GetAccount(const AccountNumber: integer): TAccount;
begin
  case AccountNumber of
    1 : result := fAccount1;
    2 : result := fAccount2;
    else raise Exception.Create('Invalid account number!');
  end;

  Console.WriteLine('A client requested account '+result.Number.ToString+
          ' ('+result.Name+')');
end;

function TBankManager.GetAccountNumbers: TAccountNumberArray;
begin
  SetLength(result, 2);
  result[0] := fAccount1.Number;
  result[1] := fAccount2.Number;

  Console.WriteLine('A client requested the list of accounts');
end;

procedure TBankManager.TransferMoney(const Origin, Destination: integer;
  const Amount: double);
var origin_acct, destination_acct : TAccount;
begin
  origin_acct := GetAccount(Origin);
  destination_acct := GetAccount(Destination);

  if (origin_acct.Balance<Amount) or (Amount<0)
    then raise Exception.Create('Insufficient funds or invalid amount specified');

  destination_acct.Balance := destination_acct.Balance+Amount;
  origin_acct.Balance := origin_acct.Balance-Amount;

  Console.WriteLine('Transferred ${0} from account {1} to account {2}',
    Amount.ToString, origin_acct.Number.ToString,
    destination_acct.Number.ToString);
end;

end.
