unit SimpleServer_Factory;

interface

uses
  SharedInterfaces,
  SimpleServer_Impl,
  System.Runtime.Remoting.Lifetime; // Contains the definition of ISponsor

type
  TSimpleFactory = class(MarshalByRefObject, ISimpleFactory, ISponsor)
  protected
    function NewInstance: ISimpleServer;
    function NewInstanceWithValue(Value: Integer): ISimpleServer;
  public
    function InitializeLifetimeService: System.Object; override;
    function Renewal(lease: ILease): TimeSpan;

  end;


implementation

{ TSimpleFactory }

function TSimpleFactory.InitializeLifetimeService: System.Object;
var lease : ILease;
begin
  Console.WriteLine('Initializing lifetime service for TSimpleFactory');

  lease := inherited InitializeLifetimeService() as ILease;

  if (lease.CurrentState = LeaseState.Initial) then begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(5);
    lease.Register(Self);
  end;

  result := lease;
end;

function TSimpleFactory.NewInstance: ISimpleServer;
begin
  result := TSimpleServer.Create;
end;

function TSimpleFactory.NewInstanceWithValue(
  Value: Integer): ISimpleServer;
begin
  result := TSimpleServer.Create(Value);
end;

function TSimpleFactory.Renewal(lease: ILease): TimeSpan;
begin
  Console.WriteLine('Request for renewal for TSimpleFactory');
  lease.Renew(TimeSpan.FromMinutes(5));
end;

end.
