unit SimpleServer_Impl;

interface

uses
  SharedInterfaces,
  System.Runtime.Remoting.Lifetime;

type
  { TSimpleServer }
  TSimpleServer = class(MarshalByRefObject, ISimpleServer, ISponsor)
  private
    fValue : integer;

  protected
    function GetValue: Integer;
    procedure SetValue(Value: Integer);

  public
    constructor Create; overload;
    constructor Create(Value : integer); overload;

    function InitializeLifetimeService: System.Object; override;
    function Renewal(Lease: ILease) : TimeSpan;

  end;

implementation

const
  LeaseDurationInMinutes = 5;

{ TSimpleServer }

constructor TSimpleServer.Create;
begin
  inherited;

  Console.WriteLine('Creating TSimpleServer with empty constructor');
end;

constructor TSimpleServer.Create(Value: integer);
begin
  inherited Create;

  fValue := Value;

  Console.WriteLine('Creating TSimpleServer with value '+Int32(Value).ToString);
end;

function TSimpleServer.GetValue: Integer;
begin
  result := fValue;

  Console.WriteLine('Getting value '+Int32(result).ToString);
end;

function TSimpleServer.InitializeLifetimeService: System.Object;
var lease : ILease;
begin
  Console.WriteLine('Initializing lifetime service for TSimpleServer');

  lease := inherited InitializeLifetimeService() as ILease;

  if (lease.CurrentState = LeaseState.Initial) then begin
    lease.InitialLeaseTime := TimeSpan.FromMinutes(LeaseDurationInMinutes);
    lease.Register(Self);
  end;

  result := lease;
end;

function TSimpleServer.Renewal(Lease: ILease): TimeSpan;
begin
  Console.WriteLine('Request for renewal for TSimpleServer');

  // Comment the following line to prevent lease renewal
  Lease.Renew(TimeSpan.FromMinutes(LeaseDurationInMinutes));
end;

procedure TSimpleServer.SetValue(Value: Integer);
begin
  fValue := Value;

  Console.WriteLine('Setting value '+Int32(Value).ToString);
end;

end.
