unit uBlockTextControl;

interface

uses
       System.Web
     , System.Web.UI
     , System.Web.UI.WebControls
     , System.ComponentModel
     ;

type
  TBlockType = (ttDivision, ttSpan, ttParagraph);

type
  [DefaultProperty('DisplayText')]
  TBlockTextWebControl = class(System.Web.UI.WebControls.WebControl)
  strict private
    FBlockType: TBlockType;
  strict protected
    function get_TagKey: HTMLTextWriterTag; override;
    procedure RenderContents(Writer: HtmlTextWriter); override;
  public
    constructor Create;
  published

    function GetDisplayText: string;
    procedure SetDisplayText(const Value: &string);
    function GetBlockType: TBlockType;
    procedure SetBlockType(const Value: TBlockType);
  published
    [
     DefaultValue(''),
     Description('The text that will be displayed in the control.'),
     Category('Appearance'),
     Bindable(True)
    ]
    property DisplayText: string read GetDisplayText write SetDisplayText;
    property BlockType: TBlockType read GetBlockType write SetBlockType;
  end;

implementation

constructor TBlockTextWebControl.Create;
begin
  inherited Create;
  ViewState['DisplayText'] := '';
  FBlockType := ttDivision;
end;

function TBlockTextWebControl.GetBlockType: TBlockType;
begin
  Result := FBlockType
end;

function TBlockTextWebControl.GetDisplayText: string;
begin
  Result := string(ViewState['DisplayText']);
end;

function TBlockTextWebControl.get_TagKey: HTMLTextWriterTag;
begin
  case FBlockType of
    ttDivision  : Result := HTMLTextWriterTag.Div;
    ttSpan      : Result := HTMLTextWriterTag.Span;
    ttParagraph : Result := HTMLTextWriterTag.P;
  else
    Result := HTMLTextWriterTag.Div;
  end;
end;

procedure TBlockTextWebControl.RenderContents(Writer: HtmlTextWriter);
begin
  Writer.Write(DisplayText);
end;

procedure TBlockTextWebControl.SetBlockType(const Value: TBlockType);
begin
  FBlockType := Value;
end;

procedure TBlockTextWebControl.SetDisplayText(const Value: &string);
begin
  ViewState['DisplayText'] := Value;
end;


end.
