unit uBoldSimpleControl;
interface

uses
       System.Web
     , System.Web.UI
     , System.ComponentModel
     ;

type
  [DefaultProperty('DisplayText')]
  TBoldTextControl = class(Control)
  private
    FUseBoldText: Boolean;
  strict protected
    procedure Render(Writer: HtmlTextWriter); override;
  public
    constructor Create;
  published
    function GetDisplayText: string;
    procedure SetDisplayText(const Value: &string);
    procedure SetUseBoldText(const Value: Boolean);
    function GetUseBoldText: Boolean;
  published
    [
     DefaultValue(''),
     Description('The text that will be displayed in the control.'),
     Category('Appearance'),
     Bindable(True)
    ]
    property DisplayText: string read GetDisplayText write SetDisplayText;
    [
     DefaultValue(False),
     Description('Determines if the text in the control will be bold.'),
     Category('Appearance'),
     Bindable(True)
    ]
    property UseBoldText: Boolean read GetUseBoldText write SetUseBoldText;
  end;

implementation

constructor TBoldTextControl.Create;
begin
  inherited Create;
  ViewState['DisplayText'] := '';
end;

function TBoldTextControl.GetDisplayText: string;
begin
  Result := string(ViewState['DisplayText']);
end;

function TBoldTextControl.GetUseBoldText: Boolean;
begin
  Result := FUseBoldText;
end;

procedure TBoldTextControl.Render(Writer: HtmlTextWriter);
begin
  if UseBoldText then
    Writer.RenderBeginTag(HTMLTextWriterTag.B);
  Writer.Write(DisplayText);
  if FUseBoldText then
    Writer.RenderEndTag;
end;

procedure TBoldTextControl.SetDisplayText(const Value: &string);
begin
  ViewState['DisplayText'] := Value;
end;

procedure TBoldTextControl.SetUseBoldText(const Value: Boolean);
begin
  FUseBoldText := Value;
end;

end.
