unit uNewUserInfoControl;

interface

uses
       System.Web
     , System.Web.UI
     , System.Web.UI.WebControls
     , System.ComponentModel
     ;

type
  TSubmitInfoEvent = procedure(Sender: TObject; e: EventArgs) of object;

type
  TNewUserInfoControl = class(WebControl, INamingContainer)
  private
    FAlignLabelsToRight: Boolean;
    FOnSubmitInfo: TSubmitInfoEvent;
    SubmitButton: Button;
    EMailLabel: System.Web.UI.WebControls.Label;
    EmailTextBox: TextBox;
    PasswordLabel: System.Web.UI.WebControls.Label;
    PasswordTextBox: TextBox;
    VerifyLabel: System.Web.UI.WebControls.Label;
    VerifyTextBox: TextBox;
    EmailValidator: RequiredFieldValidator;
    PasswordValidator: RequiredFieldValidator;
    VerifyValidator: RequiredFieldValidator;
  strict protected
    procedure Click(Sender: TObject; E: EventArgs);
    procedure Render(Writer: HTMLTextWriter); override;
    procedure CreateChildControls; override;
    procedure DoSubmitInfo(Sender: TObject; e: EventArgs); virtual;
    procedure set_AlignLabelsToRight(const Value: Boolean);
    function get_AlignLabelsToRight: Boolean;
  public
    constructor Create;
    function get_Password: string;
    function get_VerifyPassword: string;
    function get_Controls: ControlCollection; override;
    [
    DesignerSerializationVisibility
       (DesignerSerializationVisibility.Hidden)
    ]
    property Password: string read get_Password;
    [
    DesignerSerializationVisibility
       (DesignerSerializationVisibility.Hidden)
    ]
    property VerifyPassword: string read get_VerifyPassword;
    function get_ButtonText: string;
    procedure set_ButtonText(const Value: &string);
    function get_UserEmail: string;
    procedure set_UserEmail(const Value: &string);
    function get_MissingEmailErrorMessage: string;
    procedure set_MissingEmailErrorMessage(const Value: &string);
    function get_EmailLabelText: string;
    procedure set_EmailLabelText(const Value: &string);
    function get_MissingPasswordErrorMessage: string;
    procedure set_MissingPasswordErrorMessage(const Value: &string);
    function get_PasswordLabelText: string;
    procedure set_PasswordLabelText(const Value: &string);
    function get_VerifyLabelText: string;
    procedure set_MissingVerifyErrorMessage(const Value: &string);
    function get_MissingVerifyErrorMessage: string;
    procedure set_VerifyLabelText(const Value: &string);
  published
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text to be displayed on the Submit Button')
    ]
    property ButtonText: string read get_ButtonText write set_ButtonText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The user''s email address.')
    ]
    property UserEmail: string read get_UserEmail write set_UserEmail;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the email field validator')
    ]
    property MissingEmailErrorMessage: string read get_MissingEmailErrorMessage
         write set_MissingEmailErrorMessage;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
          ' email textbox')
    ]
    property EmailLabelText: string  read get_EmailLabelText
        write set_EmailLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the Password validator')
    ]
    property MissingPasswordErrorMessage: string
      read get_MissingPasswordErrorMessage write set_MissingPasswordErrorMessage;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
        ' password textbox')
    ]
    property PasswordLabelText: string read Get_PasswordLabelText
       write set_PasswordLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
        'verify password textbox')
    ]
    property VerifyLabelText: string read get_VerifyLabelText
        write set_VerifyLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the verify password validator')
    ]
    property MissingVerifyErrorMessage: string read get_MissingVerifyErrorMessage
       write set_MissingVerifyErrorMessage;
    property AlignLabelsToRight: Boolean read get_AlignLabelsToRight
       write set_AlignLabelsToRight;
    property OnSubmitInfo: TSubmitInfoEvent add FOnSubmitInfo
       remove FOnSubmitInfo;
  end;

implementation

constructor TNewUserInfoControl.Create;
begin
  inherited Create;
  FAlignLabelsToRight := True;
end;

procedure TNewUserInfoControl.CreateChildControls;
begin
  Controls.Clear;

  // Email controls
  EMailLabel := System.Web.UI.WebControls.Label.Create;

  EmailTextBox := TextBox.Create;
  EmailTextBox.ID := 'EmailTextBox';

  EmailValidator := RequiredFieldValidator.Create;
  EmailValidator.ID := 'EmailValidator';
  EmailValidator.ControlToValidate := EmailTextBox.ID;
  EmailValidator.Text := MissingEmailErrorMessage;
  EmailValidator.Display := ValidatorDisplay.Static;

  // Password Controls
  PasswordLabel := System.Web.UI.WebControls.Label.Create;

  PasswordTextBox := TextBox.Create;
  PasswordTextBox.TextMode := TextBoxMode.Password;
  PasswordTextBox.ID := 'PasswordTextBox';

  PasswordValidator := RequiredFieldValidator.Create;
  PasswordValidator.ID := 'PasswordValidator';
  PasswordValidator.ControlToValidate := PasswordTextBox.ID;
  PasswordValidator.Text := MissingPasswordErrorMessage;
  PasswordValidator.Display := ValidatorDisplay.Static;

  // VerifyPassword Controls
  VerifyLabel := System.Web.UI.WebControls.Label.Create;

  VerifyTextBox := TextBox.Create;
  VerifyTextBox.TextMode := TextBoxMode.Password;
  VerifyTextBox.ID := 'VerifyTextBox';

  VerifyValidator := RequiredFieldValidator.Create;
  VerifyValidator.ID := 'VerifyValidator';
  VerifyValidator.ControlToValidate := VerifyTextBox.ID;
  VerifyValidator.Text := MissingVerifyErrorMessage;
  VerifyValidator.Display := ValidatorDisplay.Static;

  //Submit Button
  SubmitButton := Button.Create;
  SubmitButton.ID := 'SubmitButton';
  Include(SubmitButton.Click, Self.Click);

  Controls.Add(EmailLabel);
  Controls.Add(EmailTextBox);
  Controls.Add(EmailValidator);

  Controls.Add(PasswordLabel);
  Controls.Add(PasswordTextbox);
  Controls.Add(PasswordValidator);

  Controls.Add(VerifyLabel);
  Controls.Add(VerifyTextBox);
  Controls.Add(VerifyValidator);

  Controls.Add(SubmitButton);
end;

procedure TNewUserInfoControl.Render(Writer: HTMLTextWriter);
begin
  AddAttributesToRender(Writer);

  Writer.AddAttribute(HTMLTextWriterAttribute.Cellpadding, '1', False);
  Writer.RenderBeginTag(HTMLTextWriterTag.Table);

  // Email Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
  if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  // Password Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
  if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  //Verify Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
    if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  // Submit Button
  Writer.RenderBeginTag(HTMLTextWriterTag.Tr);
  Writer.AddAttribute(HTMLTextWriterAttribute.Colspan, '2');
  Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  Writer.RenderBeginTag(HTMLTextWriterTag.Td);
  SubmitButton.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HTMLTextWriterTag.Td);
  Writer.Write('&nbsp');
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag
  Writer.RenderEndTag;  // Closes the TABLE tag

end;

function TNewUserInfoControl.get_MissingEmailErrorMessage: string;
begin
   EnsureChildControls;
   Result := EmailValidator.ErrorMessage;
end;

function TNewUserInfoControl.get_MissingPasswordErrorMessage: string;
begin
  EnsureChildControls;
  Result := PasswordValidator.ErrorMessage;
end;

function TNewUserInfoControl.get_MissingVerifyErrorMessage: string;
begin
  EnsureChildControls;
  Result := VerifyValidator.ErrorMessage;
end;

function TNewUserInfoControl.get_ButtonText: string;
begin
  EnsureChildControls;
  Result := SubmitButton.Text;
end;

function TNewUserInfoControl.get_EmailLabelText: string;
begin
  EnsureChildControls;
  Result := EMailLabel.Text;
end;

function TNewUserInfoControl.get_Password: string;
begin
  EnsureChildControls;
  Result := PasswordTextBox.Text;
end;

function TNewUserInfoControl.get_PasswordLabelText: string;
begin
  EnsureChildControls;
  Result := PasswordLabel.Text;
end;

function TNewUserInfoControl.get_UserEmail: string;
begin
  EnsureChildControls;
  Result := EmailTextBox.Text;
end;

function TNewUserInfoControl.get_VerifyLabelText: string;
begin
 EnsureChildControls;
 Result := VerifyLabel.Text;
end;

function TNewUserInfoControl.get_VerifyPassword: string;
begin
  EnsureChildControls;
  Result := VerifyTextBox.Text;
end;

function TNewUserInfoControl.get_Controls: ControlCollection;
begin
  EnsureChildControls;
  Result := inherited get_Controls;
end;

procedure TNewUserInfoControl.set_MissingEmailErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  EmailValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.set_MissingPasswordErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  PasswordValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.set_MissingVerifyErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  VerifyValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.set_ButtonText(const Value: &string);
begin
  EnsureChildControls;
  SubmitButton.Text := Value;
end;

procedure TNewUserInfoControl.set_EmailLabelText(const Value: &string);
begin
  EnsureChildControls;
  EMailLabel.Text := Value;
end;

procedure TNewUserInfoControl.set_PasswordLabelText(const Value: &string);
begin
  EnsureChildControls;
  PasswordLabel.Text := Value;
end;

procedure TNewUserInfoControl.set_UserEmail(const Value: &string);
begin
  EnsureChildControls;
  EmailTextBox.Text := Value;
end;

procedure TNewUserInfoControl.set_VerifyLabelText(const Value: &string);
begin
  EnsureChildControls;
  VerifyLabel.Text := Value;
end;

procedure TNewUserInfoControl.set_AlignLabelsToRight(const Value: Boolean);
begin
  FAlignLabelsToRight := Value;
end;

procedure TNewUserInfoControl.DoSubmitInfo(Sender: TObject; e: EventArgs);
begin
  if Assigned(FOnSubmitInfo) then
  begin
    FOnSubmitInfo(Sender, e);
  end;
end;

function TNewUserInfoControl.get_AlignLabelsToRight: Boolean;
begin
  Result := FAlignLabelsToRight;
end;

procedure TNewUserInfoControl.Click(Sender: TObject; E: EventArgs);
begin
  DoSubmitInfo(Sender, E);
end;

end.

