unit uPostBackTextWebControl;

interface

uses
       System.Web
     , System.Web.UI
     , System.Web.UI.WebControls
     , System.ComponentModel
     , System.Collections.Specialized
     ;

type
  TTextChangedEvent = procedure(e: EventArgs) of object;

type
  [DefaultProperty('DisplayText')]
  TPostBackInputWebControl = class(WebControl, IPostBackDataHandler)
  private
    FOnTextChanged: TTextChangedEvent;
    function GetDisplayText: string;
    procedure SetDisplayText(const Value: &string);
  strict protected
    procedure AddAttributesToRender(Writer: HTMLTextWriter); override;
    function get_TagKey: HTMLTextWriterTag; override;
    procedure DoTextChanged;
    procedure Render(Writer: HTMLTextWriter); override;

    // IPostBackDataHandler
    function LoadPostData(PostDataKey: string;
       PostCollection: NameValueCollection) : boolean;
    procedure RaisePostDataChangedEvent;
  public
    constructor Create;
  published
    [
     DefaultValue(''),
     Description('The text that will be displayed in the input control'),
     Category('Appearance'),
     Bindable(True)
    ]
    property DisplayText: string read GetDisplayText write SetDisplayText;
    [
    Category('Action'),
    Description('Fired whenever the user changes the text in the' +
        ' input control')
    ]
    property OnTextChanged: TTextChangedEvent
      add FOnTextChanged remove FOnTextChanged;
  end;

implementation

constructor TPostBackInputWebControl.Create;
begin
  inherited Create;
  ViewState['DisplayText'] := '';
end;

function TPostBackInputWebControl.GetDisplayText: string;
begin
  Result := string(ViewState['DisplayText']);
end;

function TPostBackInputWebControl.LoadPostData(postDataKey: &string;
  PostCollection: NameValueCollection): boolean;
var
  NewValue: string;
begin
  NewValue := PostCollection[PostDataKey];
  Result := not DisplayText.Equals(NewValue);
  if Result then
    DisplayText := NewValue;
end;

procedure TPostBackInputWebControl.RaisePostDataChangedEvent;
begin
  DoTextChanged;
end;

procedure TPostBackInputWebControl.SetDisplayText(const Value: &string);
begin
  ViewState['DisplayText'] := Value;
end;

function TPostBackInputWebControl.get_TagKey: HTMLTextWriterTag;
begin
  Result := HTMLTextWriterTag.Input;
end;

procedure TPostBackInputWebControl.DoTextChanged;
begin
  if Assigned(FOnTextChanged) then
    FOnTextChanged(EventArgs.Empty);
end;

procedure TPostBackInputWebControl.AddAttributesToRender(Writer:
     HTMLTextWriter);
begin
  inherited;
  Writer.AddAttribute(HTMLTextWriterAttribute.Name, Self.UniqueID);
  Writer.AddAttribute(HTMLTextWriterAttribute.Type, 'Text');
  Writer.AddAttribute(HTMLTextWriterAttribute.Value, DisplayText);
end;

procedure TPostBackInputWebControl.Render(Writer: HTMLTextWriter);
begin
  if Page <> nil then
    Page.VerifyRenderingInServerForm(Self);
  inherited;
end;

end.
