unit SampleControlsPage;

interface

uses
       System.Web
     , System.Web.UI
     , System.Web.UI.WebControls
     , System.ComponentModel
     ;

type
  TSubmitInfoEvent = procedure(Sender: TObject; e: EventArgs) of object;

type
  TNewUserInfoControl = class(WebControl, INamingContainer)
  private
    FAlignLabelsToRight: Boolean;
    FOnSubmitInfo: TSubmitInfoEvent;
    SubmitButton: Button;
    EMailLabel: System.Web.UI.WebControls.Label;
    EmailTextBox: TextBox;
    PasswordLabel: System.Web.UI.WebControls.Label;
    PasswordTextBox: TextBox;
    VerifyLabel: System.Web.UI.WebControls.Label;
    VerifyTextBox: TextBox;
    EmailValidator: RequiredFieldValidator;
    PasswordValidator: RequiredFieldValidator;
    VerifyValidator: RequiredFieldValidator;
  strict protected
    procedure Click(Sender: TObject; E: EventArgs);
    procedure Render(Writer: HTMLTextWriter); override;
    procedure CreateChildControls; override;
    procedure DoSubmitInfo(Sender: TObject; e: EventArgs); virtual;
    procedure SetAlignLabelsToRight(const Value: Boolean);
    function GetAlignLabelsToRight: Boolean;
  public
    function GetPassword: string;
    function GetVerifyPassword: string;
  public
    constructor Create;
    function get_Controls: ControlCollection; override;
    [
    DesignerSerializationVisibility
       (DesignerSerializationVisibility.Hidden)
    ]
    property Password: string read GetPassword;
    [
    DesignerSerializationVisibility
       (DesignerSerializationVisibility.Hidden)
    ]
    property VerifyPassword: string read GetVerifyPassword;
  published
    function GetButtonText: string;
    procedure SetButtonText(const Value: &string);
    function GetUserEmail: string;
    procedure SetUserEmail(const Value: &string);
    function GetMissingEmailErrorMessage: string;
    procedure SetMissingEmailErrorMessage(const Value: &string);
    function GetEmailLabelText: string;
    procedure SetEmailLabelText(const Value: &string);
    function GetMissingPasswordErrorMessage: string;
    procedure SetMissingPasswordErrorMessage(const Value: &string);
    function GetPasswordLabelText: string;
    procedure SetPasswordLabelText(const Value: &string);
    function GetVerifyLabelText: string;
    procedure SetMissingVerifyErrorMessage(const Value: &string);
    function GetMissingVerifyErrorMessage: string;
    procedure SetVerifyLabelText(const Value: &string);
  published
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text to be displayed on the Submit Button')
    ]
    property ButtonText: string read GetButtonText write SetButtonText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The user''s email address.')
    ]
    property UserEmail: string read GetUserEmail write SetUserEmail;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the email field validator')
    ]
    property MissingEmailErrorMessage: string read GetMissingEmailErrorMessage
         write SetMissingEmailErrorMessage;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
          ' email textbox')
    ]
    property EmailLabelText: string  read GetEmailLabelText
        write SetEmailLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the Password validator')
    ]
    property MissingPasswordErrorMessage: string
      read GetMissingPasswordErrorMessage write SetMissingPasswordErrorMessage;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
        ' password textbox')
    ]
    property PasswordLabelText: string read GetPasswordLabelText
       write SetPasswordLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The text for the label that goes with the' +
        'verify password textbox')
    ]
    property VerifyLabelText: string read GetVerifyLabelText
        write SetVerifyLabelText;
    [
      Bindable(True), Category('Appearance'), DefaultValue(''),
      Description('The error message for the verify password validator')
    ]
    property MissingVerifyErrorMessage: string read GetMissingVerifyErrorMessage
       write SetMissingVerifyErrorMessage;
    property AlignLabelsToRight: Boolean read GetAlignLabelsToRight
       write SetAlignLabelsToRight;
    property OnSubmitInfo: TSubmitInfoEvent read FOnSubmitInfo
       write FOnSubmitInfo;
  end;

implementation

constructor TNewUserInfoControl.Create;
begin
  inherited Create;
  FAlignLabelsToRight := True;
end;

procedure TNewUserInfoControl.CreateChildControls;
begin
  Controls.Clear;

  // Email controls
  EMailLabel := System.Web.UI.WebControls.Label.Create;

  EmailTextBox := TextBox.Create;
  EmailTextBox.ID := 'EmailTextBox';

  EmailValidator := RequiredFieldValidator.Create;
  EmailValidator.ID := 'EmailValidator';
  EmailValidator.ControlToValidate := EmailTextBox.ID;
  EmailValidator.Text := MissingEmailErrorMessage;
  EmailValidator.Display := ValidatorDisplay.Static;

  // Password Controls
  PasswordLabel := System.Web.UI.WebControls.Label.Create;

  PasswordTextBox := TextBox.Create;
  PasswordTextBox.TextMode := TextBoxMode.Password;
  PasswordTextBox.ID := 'PasswordTextBox';

  PasswordValidator := RequiredFieldValidator.Create;
  PasswordValidator.ID := 'PasswordValidator';
  PasswordValidator.ControlToValidate := PasswordTextBox.ID;
  PasswordValidator.Text := MissingPasswordErrorMessage;
  PasswordValidator.Display := ValidatorDisplay.Static;

  // VerifyPassword Controls
  VerifyLabel := System.Web.UI.WebControls.Label.Create;

  VerifyTextBox := TextBox.Create;
  VerifyTextBox.TextMode := TextBoxMode.Password;
  VerifyTextBox.ID := 'VerifyTextBox';

  VerifyValidator := RequiredFieldValidator.Create;
  VerifyValidator.ID := 'VerifyValidator';
  VerifyValidator.ControlToValidate := VerifyTextBox.ID;
  VerifyValidator.Text := MissingVerifyErrorMessage;
  VerifyValidator.Display := ValidatorDisplay.Static;

  //Submit Button
  SubmitButton := Button.Create;
  SubmitButton.ID := 'SubmitButton';
  Include(SubmitButton.Click, Self.Click);

  Controls.Add(EmailLabel);
  Controls.Add(EmailTextBox);
  Controls.Add(EmailValidator);

  Controls.Add(PasswordLabel);
  Controls.Add(PasswordTextbox);
  Controls.Add(PasswordValidator);

  Controls.Add(VerifyLabel);
  Controls.Add(VerifyTextBox);
  Controls.Add(VerifyValidator);

  Controls.Add(SubmitButton);
end;

procedure TNewUserInfoControl.Render(Writer: HTMLTextWriter);
begin
  AddAttributesToRender(Writer);

  Writer.AddAttribute(HTMLTextWriterAttribute.Cellpadding, '1', False);
  Writer.RenderBeginTag(HTMLTextWriterTag.Table);

  // Email Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
  if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  EmailValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  // Password Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
  if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  PasswordValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  //Verify Controls
  Writer.RenderBeginTag(HtmlTextWriterTag.Tr);
    if FAlignLabelsToRight then
  begin
    Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  end;
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyLabel.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyTextBox.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HtmlTextWriterTag.Td);
  VerifyValidator.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag

  // Submit Button
  Writer.RenderBeginTag(HTMLTextWriterTag.Tr);
  Writer.AddAttribute(HTMLTextWriterAttribute.Colspan, '2');
  Writer.AddAttribute(HTMLTextWriterAttribute.Align, 'right');
  Writer.RenderBeginTag(HTMLTextWriterTag.Td);
  SubmitButton.RenderControl(Writer);
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderBeginTag(HTMLTextWriterTag.Td);
  Writer.Write('&nbsp');
  Writer.RenderEndTag;  // Closes the TD tag
  Writer.RenderEndTag;  // Closes the TR tag
  Writer.RenderEndTag;  // Closes the TABLE tag

end;

function TNewUserInfoControl.GetMissingEmailErrorMessage: string;
begin
   EnsureChildControls;
   Result := EmailValidator.ErrorMessage;
end;

function TNewUserInfoControl.GetMissingPasswordErrorMessage: string;
begin
  EnsureChildControls;
  Result := PasswordValidator.ErrorMessage;
end;

function TNewUserInfoControl.GetMissingVerifyErrorMessage: string;
begin
  EnsureChildControls;
  Result := VerifyValidator.ErrorMessage;
end;

function TNewUserInfoControl.GetButtonText: string;
begin
  EnsureChildControls;
  Result := SubmitButton.Text;
end;

function TNewUserInfoControl.GetEmailLabelText: string;
begin
  EnsureChildControls;
  Result := EMailLabel.Text;
end;

function TNewUserInfoControl.GetPassword: string;
begin
  EnsureChildControls;
  Result := PasswordTextBox.Text;
end;

function TNewUserInfoControl.GetPasswordLabelText: string;
begin
  EnsureChildControls;
  Result := PasswordLabel.Text;
end;

function TNewUserInfoControl.GetUserEmail: string;
begin
  EnsureChildControls;
  Result := EmailTextBox.Text;
end;

function TNewUserInfoControl.GetVerifyLabelText: string;
begin
 EnsureChildControls;
 Result := VerifyLabel.Text;
end;

function TNewUserInfoControl.GetVerifyPassword: string;
begin
  EnsureChildControls;
  Result := VerifyTextBox.Text;
end;

function TNewUserInfoControl.get_Controls: ControlCollection;
begin
  EnsureChildControls;
  Result := inherited get_Controls;
end;

procedure TNewUserInfoControl.SetMissingEmailErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  EmailValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.SetMissingPasswordErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  PasswordValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.SetMissingVerifyErrorMessage(
  const Value: &string);
begin
  EnsureChildControls;
  VerifyValidator.ErrorMessage := Value;
end;

procedure TNewUserInfoControl.SetButtonText(const Value: &string);
begin
  EnsureChildControls;
  SubmitButton.Text := Value;
end;

procedure TNewUserInfoControl.SetEmailLabelText(const Value: &string);
begin
  EnsureChildControls;
  EMailLabel.Text := Value;
end;

procedure TNewUserInfoControl.SetPasswordLabelText(const Value: &string);
begin
  EnsureChildControls;
  PasswordLabel.Text := Value;
end;

procedure TNewUserInfoControl.SetUserEmail(const Value: &string);
begin
  EnsureChildControls;
  EmailTextBox.Text := Value;
end;

procedure TNewUserInfoControl.SetVerifyLabelText(const Value: &string);
begin
  EnsureChildControls;
  VerifyLabel.Text := Value;
end;

procedure TNewUserInfoControl.SetAlignLabelsToRight(const Value: Boolean);
begin
  FAlignLabelsToRight := Value;
end;

procedure TNewUserInfoControl.DoSubmitInfo(Sender: TObject; e: EventArgs);
begin
  if Assigned(FOnSubmitInfo) then
  begin
    FOnSubmitInfo(Sender, e);
  end;
end;

function TNewUserInfoControl.GetAlignLabelsToRight: Boolean;
begin
  Result := FAlignLabelsToRight;
end;

procedure TNewUserInfoControl.Click(Sender: TObject; E: EventArgs);
begin
  DoSubmitInfo(Sender, E);
end;

end.

